import json
import logging
import os
import sys
from typing import List, Tuple, Union

import requests
import urllib3
from deepsecurity.rest import ApiException

from dsmigrator.logging import log


def safe_list_get(l: list, idx: int):
    try:
        return l[idx]
    except IndexError:
        return None


def rename_json(json: dict) -> dict:
    """
    Adds 'Migrated' to the end of items

    Args:
        json (dict): dict of some kind from json

    Returns:
        dict: same dict with different 'name' key
    """
    if json["name"]:
        json["name"] = f"{json['name']} - Migrated"
    return json


def to_title(snake_case: str) -> str:
    """
    Takes a snake case string and turns it into a title case string

    Args:
        snake_case (str): snake case string

    Returns:
        str: title case string
    """
    temp = str(snake_case).split("_")
    title = " ".join(ele.title() for ele in temp)
    return title


def safe_request(
    old_api_key: str, http_method: str, url: str, payload: dict, cert: Union[str, bool]
) -> requests.models.Response:
    """
    Makes a request through the requests module, and handles errors through rich

    Args:
        old_api_key (str): self-explanatory
        http_method (str): "GET", "POST", etc
        url (str): of form "https://www.google.com/"
        payload (dict): Dict version of json payload
        cert (Union[str, bool]): if using a cert, pass string file path here. If not, pass False.

    Returns:
        requests.models.Response: Response object from Requests
    """
    headers = {
        "api-secret-key": old_api_key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    try:
        return requests.request(
            http_method,
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
    except Exception as e:
        log.exception(e)
        log.critical(
            "Having trouble connecting to the old DSM. Please ensure the url and routes are correct."
        )
        log.critical("Aborting...")
        sys.exit(0)


def validate_create(all_old: List[str], api_instance, type: str) -> List[str]:
    """
    Takes in old json objects, creates them in Cloud one, and returns a list of
    the new object IDs in Cloud One

    Args:
        all_old (List[str]): list of string-formatted json objects from DS
        api_instance (RestApiConfiguration): SDK module wrapper object configured from api_config
        type (str): what to display during printing (e.g. 'Antimalware Configuration')

    Returns:
        List[str]: A list of all the objects' new IDs in Cloud One (as strings)
    """
    all_new = []
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson["name"]
        while namecheck != -1:
            try:
                newname = api_instance.create(oldjson)
                newid = api_instance.search(newname)
                log.info(
                    "#"
                    + str(count)
                    + " "
                    + type.capitalize()
                    + " ID: "
                    + str(newid)
                    + ", Name: "
                    + newname,
                )
                all_new.append(str(newid))
                namecheck = -1
            except ApiException as e:
                error_json = json.loads(e.body)
                if "name already exists" in error_json["message"]:
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                elif "Name must be unique" in error_json["message"]:
                    log.info(
                        f"{oldjson['name']} already exists in new tenant, renaming...",
                    )
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    log.info(e.body)
                    namecheck = -1
    return all_new


def validate_create_dict(all_old: list, api_instance, type: str) -> dict:
    """Take in a list of old objects, create the objects in Cloud One, then make a mapping of oldids to new ids for use later

    Args:
        all_old (list): a list of json strings that are provided as output of other functions
        api_instance (RestApiConfiguration): an sdk configuration from api_config
        type (str): The type of operation being performed, for log.infoing (e.g. "Intrusion Prevention Rule")

    Returns:
        dict: A dict of form {oldid1:newid1, oldid2:newid2, ...}
    """
    new_dict = {}
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson.get("name")
        oldid = oldjson.get("ID")
        if (oldname is not None) and (oldid is not None):
            while namecheck != -1:
                try:
                    newname = api_instance.create(oldjson)
                    newid = api_instance.search(newname)
                    log.info(
                        "#"
                        + str(count)
                        + " "
                        + type.capitalize()
                        + " List ID: "
                        + str(newid)
                        + ", Name: "
                        + newname,
                    )
                    new_dict[oldid] = newid
                    namecheck = -1
                except ApiException as e:
                    error_json = json.loads(e.body)
                    if "name already exists" in error_json["message"]:
                        log.info(
                            f"{oldjson['name']} already exists in new tenant, renaming..."
                        )
                        oldjson["name"] = oldname + " {" + str(rename) + "}"
                        rename = rename + 1
                    else:
                        log.info(e.body)
                        namecheck = -1
    return new_dict


def validate_create_dict_custom(
    all_old: list, skeleton_dict: dict, api_instance, type: str
) -> Tuple[dict, list]:
    """Transforms a list of pre-defined DS objects in json into a id mapping dict and a list of custom items

    Args:
        all_old (list): list of pre-defined objects as json strings
        skeleton_dict (dict): A skeleton of the final ID mapping of form {oldid1: None, oldid2: None,...}
        api_instance (ApiInstance): The requisite ApiInstance object defined in api_config
        type (str): The type of operation being performed, for log.infoing (e.g. "Intrusion Prevention Rule")

    Returns:
        Tuple[(dict, list)]: A dict of form {oldid:newid,...} for all common pre-defined objects, and a list of custom objects as strings of json
    """
    custom_list = []
    for (count, json_string) in enumerate(all_old):
        namecheck = 1
        rename = 1
        item_json = json.loads(json_string)
        old_id = item_json["ID"]
        old_name = item_json["name"]
        while namecheck != -1:
            try:
                new_id = api_instance.search(old_name)
                if new_id is not None:
                    skeleton_dict[old_id] = new_id
                    log.info(
                        f"#{str(count)} {type}: {old_name}",
                    )
                elif "template" in item_json:
                    custom_list.append(json.dumps(item_json))
                else:
                    log.warning(
                        f"{old_name} is not valid in Workload Security and will not be transferred."
                    )
                    log.warning(
                        f"Consider transferring {old_name} manually using the xml export feature in the GUI."
                    )
                namecheck = -1
            except ApiException as e:
                if "already exists" in e.body:
                    log.info(f"{old_name} already exists in new tenant, renaming...")
                    item_json["name"] = old_name + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    log.error(e.body)
                    pass
    log.info("Done!")
    return skeleton_dict, custom_list



import json
import typing
import os
import sys
from types import SimpleNamespace

import deepsecurity
import requests
import urllib3
from deepsecurity.rest import ApiException
from rich.progress import Progress

from dsmigrator.api_config import PolicyApiInstance
from dsmigrator.migrator_utils import safe_request
from dsmigrator.logging import log

cert = False


def delete_cloud_one_policies(CLOUD_ONE_API_KEY: str):
    log.info("Deleting Policies from Cloud One...")

    host = "https://cloudone.trendmicro.com/api/policies"
    message = requests.get(
        host,
        verify=False,
        headers={"api-secret-key": CLOUD_ONE_API_KEY, "api-version": "v1"},
    )
    parsed = message.text
    x = json.loads(parsed, object_hook=lambda d: SimpleNamespace(**d))
    policyNumbers = []
    for key in x.policies:  # ERRORS here
        policyNumbers.append(key.ID)
    if policyNumbers:
        for policyID in policyNumbers:
            log.info(f"Deleting policy #{policyID}")
            policyHost = f"https://cloudone.trendmicro.com/api/policies/{policyID}"
            requests.delete(
                policyHost,
                verify=False,
                headers={"api-secret-key": CLOUD_ONE_API_KEY, "api-version": "v1"},
            )
    else:
        log.info("No policies detected in Cloud One. Skipping.")


def ListAllPolicy(url_link_final, tenant1key):
    payload = {}
    url = url_link_final + "api/policies"
    response = safe_request(tenant1key, "GET", url, payload=payload, cert=cert)
    describe = str(response.text)
    oldpolicynameid_dict = {}
    oldpolicyid = []
    namejson = json.loads(describe)
    for policy in namejson["policies"]:
        oldpolicynameid_dict[str(policy["name"])] = str(policy["ID"])
        oldpolicyid.append(str(policy["ID"]))
    return oldpolicyid, oldpolicynameid_dict


def GetPolicy(policyIDs, url_link_final, tenant1key):
    antimalwareconfig = []
    allofpolicy = []
    i = 0
    log.info("Getting Policy from Deep Security")
    for count, part in enumerate(policyIDs):
        payload = {}
        url = url_link_final + "api/policies/" + str(part)
        response = safe_request(tenant1key, "GET", url, payload=payload, cert=cert)
        policy_string = str(response.text)
        i = i + 1
        allofpolicy.append(policy_string)
        log.info("#" + str(count) + " Policy ID: " + str(part))
        rtscan = policy_string.find("realTimeScanConfigurationID")
        if rtscan != -1:
            rtpart = policy_string[rtscan + 28 :]
            startIndex = rtpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = rtpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    rtid = rtpart[startIndex + 1 : endIndex]
                    antimalwareconfig.append(str(rtid))

        mscan = policy_string.find("manualScanConfigurationID")
        if mscan != -1:
            mpart = policy_string[mscan + 26 :]
            startIndex = mpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = mpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    mid = mpart[startIndex + 1 : endIndex]
                    antimalwareconfig.append(str(mid))

        sscan = policy_string.find("scheduledScanConfigurationID")
        if sscan != -1:
            spart = policy_string[sscan + 29 :]
            startIndex = spart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = spart.find("}", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    ssid = spart[startIndex + 1 : endIndex]
                    antimalwareconfig.append(str(ssid))
    antimalwareconfig = list(dict.fromkeys(antimalwareconfig))
    return antimalwareconfig, allofpolicy


def policy_validate_create(
    all_old: typing.List[str], api_instance: PolicyApiInstance, type: str
) -> typing.Dict[int, int]:
    """
    Takes in a list of json strings that represent policies, sets some key
    settings, assigns the proper parent ID to each, creates each policy,
    renames if necessary, and outputs a dictionary of form
    {oldid: newid, ...} for future use.

    Args:
        all_old (typing.List[str]): A list containing strings of json data
                                    representing policies
        api_instance (PolicyApiInstance): From api_config, a wrapper SDK class
        type (str): The type of output for printing purposes

    Returns:
        typing.Dict[int, int]: dict of form {oldid:newid, ...}
    """
    all_new = []
    id_dict = {}
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson["name"]
        oldid = oldjson["ID"]
        # cleanup input
        if "policySettings" in oldjson.keys():
            oldjson["policySettings"]["platformSettingAgentCommunicationsDirection"] = {
                "value": "Agent/Appliance Initiated"
            }
        # Modify parent ID. If errors, to prevent wasted time, create policy at base level.
        try:
            if "parentID" in oldjson.keys():
                oldjson["parentID"] = id_dict[oldjson["parentID"]]
        except Exception as e:
            log.exception(e)
            log.warning(
                "ParentID error. Creating policy at base level, please set inheritance manually."
            )
            del oldjson["parentID"]
            pass
        while namecheck != -1:
            try:
                newname = api_instance.create(oldjson)
                newid = api_instance.search(newname).id
                id_dict[oldid] = newid
                log.info(
                    "#"
                    + str(count)
                    + " "
                    + type.capitalize()
                    + " List ID: "
                    + str(newid)
                    + ", Name: "
                    + newname,
                )
                all_new.append(str(newid))
                namecheck = -1
            except ApiException as e:
                error_json = json.loads(e.body)
                if "name already exists" in error_json["message"]:
                    log.info(
                        f"{oldjson['name']} already exists in new tenant, renaming..."
                    )
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    log.exception(e)
                    log.error(
                        f"{oldname} could not be transferred. Please transfer manually."
                    )
                    namecheck = -1
    id_dict[0] = 0
    return id_dict


cert = False


def AddPolicy(allofpolicy: typing.List[str], NEW_API_KEY: str) -> typing.Dict[int, int]:
    log.info("Creating Policy to Tenant 2 with new ID")
    if allofpolicy:
        return policy_validate_create(
            allofpolicy, PolicyApiInstance(NEW_API_KEY), "policy"
        )



import logging
from datetime import datetime
from rich.console import Console
from rich.logging import RichHandler


console = Console(log_path=False)
error_console = True

LOG_FILE_FORMAT = logging.Formatter("%(asctime)s — %(levelname)s — %(message)s")
LOG_FILE = datetime.now().strftime("dsmigrator_%H_%M_%d_%m_%Y.log")


def get_file_handler():
    file_handler = logging.FileHandler(LOG_FILE)
    file_handler.setFormatter(LOG_FILE_FORMAT)
    return file_handler


def get_console_handler():
    stream_handler = RichHandler(console=console, show_path=False)
    stream_handler.setLevel(logging.INFO)
    return stream_handler


logging.basicConfig(
    level="DEBUG",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[
        get_console_handler(),
        get_file_handler(),
    ],
)

log = logging.getLogger("dsmigrator")



# from __future__ import log.info_function
from pathlib import Path
import click
import sys, warnings
import deepsecurity
from deepsecurity.rest import ApiException
import json
from dsmigrator.logging import log
import xml.etree.ElementTree as ET


@click.command()
@click.option(
    "-x",
    "--xml-folder",
    prompt="Path to XML folder with IM, LI, and IPS rule export files",
    help="Path to XML folder with IM, LI, and IPS rule export files. NOTE that the files must be named 'Intrusion_Prevention_Rules.xml', 'Log_Inspection_Rules.xml', and 'Integrity_Monitoring_Rules.xml'",
    default="./xml/",
)
@click.option(
    "-ou",
    "--original-url",
    prompt="Old DSM url",
    help="A resolvable FQDN for the old DSM, with port number (e.g. https://192.168.1.1:4119)",
    envvar="ORIGINAL_URL",
)
@click.option(
    "-oa",
    "--original-api-key",
    prompt="Old DSM API key",
    hide_input=True,
    help="API key for the old DSM with Full Access permissions",
    envvar="ORIGINAL_API_KEY",
)
@click.option(
    "--outfile",
    help="Path to output json file",
    default="./CUSTOMIZED_RULES.json",
)
@click.option(
    "-k",
    "--insecure",
    is_flag=True,
    help="Suppress the InsecureRequestWarning for self-signed certificates",
)
@click.option(
    "-c",
    "--cert",
    default=False,
    show_default=True,
    help="(Optional) Allows the use of a cert file",
)
def main(xml_folder, outfile, original_url, original_api_key, insecure, cert=False):
    log.info("Searching for customized rules...")

    class ApiInstance:
        def __init__(self, module, ORIGINAL_URL, ORIGINAL_API_KEY, cert=False):
            self.module = module
            self.configuration = deepsecurity.Configuration()
            self.configuration.host = f"{ORIGINAL_URL}/api"
            self.configuration.api_key["api-secret-key"] = ORIGINAL_API_KEY
            self.api_version = "v1"

        def search(self, search_filter):
            if self.module == "Intrusion Prevention":
                api_instance = deepsecurity.IntrusionPreventionRulesApi(
                    deepsecurity.ApiClient(self.configuration)
                )
                return api_instance.search_intrusion_prevention_rules(
                    self.api_version, search_filter=search_filter
                ).intrusion_prevention_rules
            elif self.module == "Log Inspection":
                api_instance = deepsecurity.LogInspectionRulesApi(
                    deepsecurity.ApiClient(self.configuration)
                )
                return api_instance.search_log_inspection_rules(
                    self.api_version, search_filter=search_filter
                ).log_inspection_rules
            elif self.module == "Integrity Monitoring":
                api_instance = deepsecurity.IntegrityMonitoringRulesApi(
                    deepsecurity.ApiClient(self.configuration)
                )
                return api_instance.search_integrity_monitoring_rules(
                    self.api_version, search_filter=search_filter
                ).integrity_monitoring_rules

    class EditedRules(ApiInstance):
        def __init__(self, file, module):
            self.file = file
            self.module = module
            self.xml = ET.parse(file).getroot()
            self.edited_rules = []
            iterator = {
                "Intrusion Prevention": "PayloadFilter2",
                "Integrity Monitoring": "IntegrityRule",
                "Log Inspection": "LogInspectionRule",
            }
            for rule in self.xml.iter(iterator[self.module]):
                for child in rule.iter("UserEdited"):
                    if child.text == "true":
                        rule_id = rule.attrib["id"]
                        self.edited_rules.append(int(rule_id))
            if len(self.edited_rules) > 1:
                log.info(
                    f"Found {len(self.edited_rules)} customized {self.module} rules!"
                )
            elif self.edited_rules:
                log.info(
                    f"Found {len(self.edited_rules)} customized {self.module} rule!"
                )
            else:
                log.info(f"No customized {self.module} rules detected.")

    def add_module_rules(api_instance, edited_rules_list):
        rule_list = {
            "module": api_instance.module,
            "date": edited_rules_list.xml.attrib["date"],
            "rules": [],
        }
        for edited_rule in edited_rules_list.edited_rules:
            search_criteria = deepsecurity.SearchCriteria()
            search_criteria.id_value = edited_rule
            search_criteria.id_test = "equal"
            search_filter = deepsecurity.SearchFilter()
            search_filter.search_criteria = [search_criteria]
            rule = api_instance.search(search_filter=search_filter)[0]
            rule_list["rules"].append(
                {
                    "identifier": int(rule.identifier),
                    "details": {
                        "name": rule.name,
                        "description": rule.description,
                    },
                }
            )
        return rule_list

    class Config(ApiInstance):
        def __init__(
            self, module, rules_file, ORIGINAL_URL, ORIGINAL_API_KEY, cert=False
        ):
            self.api = ApiInstance(module, ORIGINAL_URL, ORIGINAL_API_KEY, cert)
            self.rules = EditedRules(rules_file, module)

    if not sys.warnoptions:
        warnings.simplefilter("ignore")

    ip = Config(
        "Intrusion Prevention",
        f"{xml_folder}Intrusion_Prevention_Rules.xml",
        original_url,
        original_api_key,
        cert,
    )
    im = Config(
        "Integrity Monitoring",
        f"{xml_folder}Integrity_Monitoring_Rules.xml",
        original_url,
        original_api_key,
        cert,
    )
    li = Config(
        "Log Inspection",
        f"{xml_folder}Log_Inspection_Rules.xml",
        original_url,
        original_api_key,
        cert,
    )
    log.info(f"Creating report at {outfile}...")
    try:
        rule_list = [
            add_module_rules(ip.api, ip.rules),
            add_module_rules(im.api, im.rules),
            add_module_rules(li.api, li.rules),
        ]
    except ApiException as e:
        log.info(e)
    json_output = json.dumps(rule_list, indent=2)
    open(outfile, "w").write(json_output)
    log.info("DONE!")


if __name__ == "__main__":
    main()  # pylint: disable=no-value-for-parameter



import sys
import os
import time
from time import sleep
import requests
import urllib3
import urllib3
import json
from dsmigrator.logging import log
from dsmigrator.api_config import (
    DirectoryListsApiInstance,
    FileListsApiInstance,
    FileExtensionListsApiInstance,
)
from dsmigrator.migrator_utils import (
    validate_create,
    rename_json,
    safe_request,
)

cert = False


def directory_listmaker(
    amdirectorylist, amfileextensionlist, amfilelist, OLD_HOST, OLD_API_KEY, NEW_API_KEY
):
    og_alldirectory = DirListTenant1(amdirectorylist, OLD_HOST, OLD_API_KEY)
    og_allfileextension = FileExtensionListTenant1(
        amfileextensionlist, OLD_HOST, OLD_API_KEY
    )
    og_allfilelist = FileListTenant1(amfilelist, OLD_HOST, OLD_API_KEY)

    alldirectory, allfilelist, allfileextension = RenameLists(
        og_alldirectory, og_allfilelist, og_allfileextension
    )

    amalldirectorynew = DirListTenant2(alldirectory, NEW_API_KEY)
    amallfileextensionnew = FileExtensionListTenant2(allfileextension, NEW_API_KEY)
    amallfilelistnew = FileListTenant2(allfilelist, NEW_API_KEY)

    return (
        amalldirectorynew,
        amallfileextensionnew,
        amallfilelistnew,
    )


def port_listmaker(
    OLD_HOST,
    OLD_API_KEY,
    NEW_HOST,
    NEW_API_KEY,
):
    t1portlistall, t1portlistname, t1portlistid = PortListGet(OLD_HOST, OLD_API_KEY)
    t2portlistid = PortListCreate(t1portlistall, t1portlistname, NEW_HOST, NEW_API_KEY)
    return t1portlistid, t2portlistid


def mac_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    t1maclistall, t1maclistname, t1maclistid = MacListGet(OLD_HOST, OLD_API_KEY)
    t2maclistid = MacListCreate(t1maclistall, t1maclistname, NEW_HOST, NEW_API_KEY)
    return t1maclistid, t2maclistid


def ip_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    t1iplistall, t1iplistname, t1iplistid = IpListGet(OLD_HOST, OLD_API_KEY)
    t2iplistid = IpListCreate(t1iplistall, t1iplistname, NEW_HOST, NEW_API_KEY)
    return t1iplistid, t2iplistid


def stateful_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    t1statefulall, t1statefulname, t1statefulid = StatefulGet(OLD_HOST, OLD_API_KEY)
    t2statefulid = StatefulCreate(t1statefulall, t1statefulname, NEW_HOST, NEW_API_KEY)
    stateful_dict = {0: 0}
    for i in t1statefulid:
        index = t1statefulid.index(i)
        if index < len(t2statefulid):
            stateful_dict[int(i)] = int(t2statefulid[index])
    return t1statefulid, t2statefulid, stateful_dict


def context_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    t1contextall, t1contextname, t1contextid = ContextGet(OLD_HOST, OLD_API_KEY)
    t2contextid = ContextCreate(t1contextall, t1contextname, NEW_HOST, NEW_API_KEY)
    return t1contextid, t2contextid


def schedule_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    t1scheduleall, t1schedulename, t1scheduleid = ScheduleGet(OLD_HOST, OLD_API_KEY)
    schedule_id_dict, t2scheduleid = ScheduleCreate(
        t1scheduleall, t1schedulename, NEW_HOST, NEW_API_KEY
    )
    return schedule_id_dict, t1scheduleid, t2scheduleid


def DirListTenant1(directorylist, url_link_final, tenant1key):
    alldirectory = []
    log.info("Getting lists from Tenant 1, if any.")
    for dirlist in directorylist:
        payload = {}
        url = url_link_final + "api/directorylists/" + str(dirlist)
        headers = {
            "api-secret-key": tenant1key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "GET",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        alldirectory.append(describe)
    log.info("Tenant1 directory list")
    log.info(directorylist)
    return alldirectory


def FileExtensionListTenant1(fileextentionlist, url_link_final, tenant1key):
    allfileextention = []
    for dirlist in fileextentionlist:
        payload = {}
        url = url_link_final + "api/fileextensionlists/" + str(dirlist)
        headers = {
            "api-secret-key": tenant1key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "GET",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        allfileextention.append(describe)
    log.info("Tenant1 file extention list")
    log.info(fileextentionlist)
    return allfileextention


def FileListTenant1(filelist, url_link_final, tenant1key):
    allfilelist = []
    for dirlist in filelist:
        payload = {}
        url = url_link_final + "api/filelists/" + str(dirlist)
        headers = {
            "api-secret-key": tenant1key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "GET",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        allfilelist.append(describe)
    log.info("Tenant1 file list")
    log.info(filelist)
    return allfilelist


def RenameLists(alldirectory, allfilelist, allfileextention):
    count = 0
    if alldirectory:
        for describe in alldirectory:
            descjson = json.loads(describe)
            descjson["name"] = descjson["name"] + " - Migrated"
            alldirectory[count] = json.dumps(descjson)
            count = count + 1

    count = 0
    if allfilelist:
        for describe in allfilelist:
            descjson = json.loads(describe)
            descjson["name"] = descjson["name"] + " - Migrated"
            allfilelist[count] = json.dumps(descjson)
            count = count + 1
    count = 0
    if allfileextention:
        for describe in allfileextention:
            descjson = json.loads(describe)
            descjson["name"] = descjson["name"] + " - Migrated"
            allfileextention[count] = json.dumps(descjson)
            count = count + 1
    return alldirectory, allfilelist, allfileextention


def DirListTenant2(alldirectory, NEW_API_KEY):
    log.info("Creating directory list in tenant 2, if any")
    alldirectorynew = []
    if alldirectory:
        alldirectorynew = validate_create(
            alldirectory, DirectoryListsApiInstance(NEW_API_KEY), "directory"
        )
    log.info(alldirectorynew)
    return alldirectorynew


def FileListTenant2(allfile, NEW_API_KEY):
    log.info("Creating file list in tenant 2, if any")
    allfilenew = []
    if allfile:
        allfilenew = validate_create(allfile, FileListsApiInstance(NEW_API_KEY), "file")
    log.info("new file list")
    log.info(allfilenew)
    return allfilenew


def FileExtensionListTenant2(allfileext, NEW_API_KEY):
    log.info("Creating file extension list in tenant 2, if any")
    allfileextnew = []
    if allfileext:
        allfileextnew = validate_create(
            allfileext, FileExtensionListsApiInstance(NEW_API_KEY), "file extension"
        )
    log.info("new file extension list")
    log.info(allfileextnew)
    return allfileextnew


def PortListGet(url_link_final, tenant1key):
    t1portlistall = []
    t1portlistname = []
    t1portlistid = []
    log.info("Getting All Port List...")
    url = url_link_final + "api/portlists"
    response = safe_request(tenant1key, "GET", url, payload={}, cert=cert)
    describe = str(response.text)
    ports_json = json.loads(describe).get("portLists")
    if ports_json is not None:
        for count, here in enumerate(ports_json):
            t1portlistall.append(str(json.dumps(here)))
            t1portlistname.append(str(here["name"]))
            log.info("#" + str(count) + " Port List name: " + str(here["name"]))
            t1portlistid.append(str(here["ID"]))
            log.info("#" + str(count) + " Port List ID: " + str(here["ID"]))
        log.info("Done!")
    return t1portlistall, t1portlistname, t1portlistid


def PortListCreate(t1portlistall, t1portlistname, url_link_final_2, tenant2key):
    t2portlistid = []
    log.info("Transfering All Port List...")
    for count, dirlist in enumerate(t1portlistname):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/portlists/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        index = describe.find(dirlist)
        if index != -1:
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        payload = t1portlistall[count]
                        url = url_link_final_2 + "api/portlists/" + str(indexid)
                        headers = {
                            "api-secret-key": tenant2key,
                            "api-version": "v1",
                            "Content-Type": "application/json",
                        }
                        response = requests.request(
                            "POST",
                            url,
                            headers=headers,
                            data=payload,
                            verify=cert,
                        )
                        t2portlistid.append(str(indexid))
                        log.info("#" + str(count) + " Port List ID: " + indexid)
        else:
            payload = t1portlistall[count]
            url = url_link_final_2 + "api/portlists"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        t2portlistid.append(str(indexid))
                        log.info("#" + str(count) + " Port List ID:" + indexid)
            else:
                log.info(describe)
                log.info(payload)
    log.info("Done!")
    return t2portlistid


def MacListGet(url_link_final, tenant1key):
    t1maclistall = []
    t1maclistname = []
    t1maclistid = []
    log.info("Getting All Mac List...")
    url = url_link_final + "api/maclists"
    response = safe_request(tenant1key, "GET", url, payload={}, cert=cert)
    describe = str(response.text)
    mac_json = json.loads(describe).get("macLists")
    if mac_json is not None:
        for count, here in enumerate(mac_json):
            t1maclistall.append(json.dumps(here))
            t1maclistname.append(str(here["name"]))
            log.info("#" + str(count) + " Mac List name: " + str(here["name"]))
            t1maclistid.append(str(here["ID"]))
            log.info("#" + str(count) + " Mac List ID: " + str(here["ID"]))

        log.info("Done!")
    return t1maclistall, t1maclistname, t1maclistid


def MacListCreate(t1maclistall, t1maclistname, url_link_final_2, tenant2key):
    t2maclistid = []
    log.info("Transfering All Mac List...")
    for count, dirlist in enumerate(t1maclistname):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/maclists/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        index = describe.find(dirlist)
        if index != -1:
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        payload = t1maclistall[count]
                        url = url_link_final_2 + "api/maclists/" + str(indexid)
                        headers = {
                            "api-secret-key": tenant2key,
                            "api-version": "v1",
                            "Content-Type": "application/json",
                        }
                        response = requests.request(
                            "POST",
                            url,
                            headers=headers,
                            data=payload,
                            verify=cert,
                        )
                        t2maclistid.append(str(indexid))
                        log.info("#" + str(count) + " MAC List ID: " + indexid)
        else:
            payload = t1maclistall[count]
            url = url_link_final_2 + "api/maclists"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        t2maclistid.append(str(indexid))
                        log.info("#" + str(count) + " MAC List ID: " + indexid)
            else:
                log.info(describe)
                log.info(payload)
    log.info("Done!")
    return t2maclistid


def IpListGet(url_link_final, tenant1key):
    t1iplistall = []
    t1iplistname = []
    t1iplistid = []
    log.info("Getting All IP List...")
    payload = {}
    url = url_link_final + "api/iplists"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request(
        "GET",
        url,
        headers=headers,
        data=payload,
        verify=cert,
    )
    describe = str(response.text)
    ip_json = json.loads(describe).get("ipLists")
    if ip_json:
        for count, here in enumerate(ip_json):
            t1iplistall.append(json.dumps(here))
            t1iplistname.append(str(here["name"]))
            log.info("#" + str(count) + " IP List name: " + str(here["name"]))
            t1iplistid.append(str(here["ID"]))
            log.info("#" + str(count) + " IP List ID: " + str(here["ID"]))
        log.info("Done!")
    return t1iplistall, t1iplistname, t1iplistid


def IpListCreate(t1iplistall, t1iplistname, url_link_final_2, tenant2key):
    t2iplistid = []
    log.info("Transfering All IP List...")
    for count, dirlist in enumerate(t1iplistname):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/iplists/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        index = describe.find(dirlist)
        if index != -1:
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        payload = t1iplistall[count]
                        url = url_link_final_2 + "api/iplists/" + str(indexid)
                        headers = {
                            "api-secret-key": tenant2key,
                            "api-version": "v1",
                            "Content-Type": "application/json",
                        }
                        response = requests.request(
                            "POST",
                            url,
                            headers=headers,
                            data=payload,
                            verify=cert,
                        )
                        t2iplistid.append(str(indexid))
                        log.info("#" + str(count) + " IP List ID: " + indexid)
        else:
            payload = t1iplistall[count]
            url = url_link_final_2 + "api/iplists"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        t2iplistid.append(str(indexid))
                        log.info("#" + str(count) + " IP List ID: " + indexid)
            else:
                log.info(describe)
                log.info(payload)
    log.info("Done!")
    return t2iplistid


def StatefulGet(url_link_final, tenant1key):
    t1statefulall = []
    t1statefulname = []
    t1statefulid = []
    log.info("Getting All Stateful Configuration...")
    payload = {}
    url = url_link_final + "api/statefulconfigurations"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request(
        "GET",
        url,
        headers=headers,
        data=payload,
        verify=cert,
    )
    describe = str(response.text)
    stateful_json = json.loads(describe).get("statefulConfigurations")
    if stateful_json is not None:
        for count, here in enumerate(stateful_json):
            t1statefulall.append(str(json.dumps(here)))
            t1statefulname.append(str(here["name"]))
            log.info(
                "#" + str(count) + " Stateful Config name: " + str(here["name"]),
            )
            t1statefulid.append(str(here["ID"]))
            log.info("#" + str(count) + " Stateful Config ID: " + str(here["ID"]))
        log.info("Done")
    return t1statefulall, t1statefulname, t1statefulid


def StatefulCreate(t1statefulall, t1statefulname, url_link_final_2, tenant2key):
    t2statefulid = []
    log.info("Transfering All Stateful Configuration...")
    if t1statefulname:
        for count, dirlist in enumerate(t1statefulname):
            payload = (
                '{"searchCriteria": [{"fieldName": "name","stringValue": "'
                + dirlist
                + '"}]}'
            )
            url = url_link_final_2 + "api/statefulconfigurations/search"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            taskjson = json.loads(describe)
            if not "message" in taskjson:
                if taskjson["statefulConfigurations"]:
                    for here in taskjson["statefulConfigurations"]:
                        indexid = here["ID"]
                        payload = t1statefulall[count]
                        url = (
                            url_link_final_2
                            + "api/statefulconfigurations/"
                            + str(indexid)
                        )
                        headers = {
                            "api-secret-key": tenant2key,
                            "api-version": "v1",
                            "Content-Type": "application/json",
                        }
                        response = requests.request(
                            "POST",
                            url,
                            headers=headers,
                            data=payload,
                            verify=cert,
                        )
                        describe = str(response.text)
                        taskjson1 = json.loads(describe)
                        t2statefulid.append(str(taskjson1["ID"]))
                        log.info(
                            "#"
                            + str(count)
                            + " Stateful Config name: "
                            + taskjson1["name"],
                        )
                        log.info(
                            "#"
                            + str(count)
                            + " Stateful Config ID: "
                            + str(taskjson1["ID"]),
                        )
                else:
                    payload = t1statefulall[count]
                    url = url_link_final_2 + "api/statefulconfigurations"
                    headers = {
                        "api-secret-key": tenant2key,
                        "api-version": "v1",
                        "Content-Type": "application/json",
                    }
                    response = requests.request(
                        "POST",
                        url,
                        headers=headers,
                        data=payload,
                        verify=cert,
                    )
                    describe = str(response.text)
                    taskjson = json.loads(describe)
                    t2statefulid.append(str(taskjson["ID"]))
                    log.info(
                        "#" + str(count) + " Stateful Config name: " + taskjson["name"],
                    )
                    log.info(
                        "#"
                        + str(count)
                        + " Stateful Config ID: "
                        + str(taskjson["ID"]),
                    )
            else:
                log.info(describe)
                log.info(payload)
    log.info("Done!")
    return t2statefulid


def ContextGet(url_link_final, tenant1key):
    t1contextall = []
    t1contextname = []
    t1contextid = []
    log.info("Getting All Context Configuration...")
    payload = {}
    url = url_link_final + "api/contexts"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request(
        "GET",
        url,
        headers=headers,
        data=payload,
        verify=cert,
    )
    describe = str(response.text)
    contexts_json = json.loads(describe).get("contexts")
    if contexts_json is not None:
        for count, here in enumerate(contexts_json):
            t1contextall.append(str(json.dumps(here)))
            t1contextname.append(str(here["name"]))
            log.info(
                "#" + str(count) + " Context Config name: " + str(here["name"]),
            )
            t1contextid.append(str(here["ID"]))
            log.info("#" + str(count) + " Context Config ID: " + str(here["ID"]))
        log.info("Done")
    return t1contextall, t1contextname, t1contextid


def ContextCreate(t1contextall, t1contextname, url_link_final_2, tenant2key):
    t2contextid = []
    log.info("Transfering All Context Configuration...")
    if t1contextname:
        for count, dirlist in enumerate(t1contextname):
            payload = (
                '{"searchCriteria": [{"fieldName": "name","stringValue": "'
                + dirlist
                + '"}]}'
            )
            url = url_link_final_2 + "api/contexts/search"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            taskjson = json.loads(describe)
            if not "message" in taskjson:
                if taskjson["contexts"]:
                    for here in taskjson["contexts"]:
                        indexid = here["ID"]
                        payload = t1contextall[count]
                        url = url_link_final_2 + "api/contexts/" + str(indexid)
                        headers = {
                            "api-secret-key": tenant2key,
                            "api-version": "v1",
                            "Content-Type": "application/json",
                        }
                        response = requests.request(
                            "POST",
                            url,
                            headers=headers,
                            data=payload,
                            verify=cert,
                        )
                        describe = str(response.text)
                        taskjson1 = json.loads(describe)
                        t2contextid.append(str(taskjson1["ID"]))
                        log.info(
                            "#"
                            + str(count)
                            + " Context Config name: "
                            + taskjson1["name"],
                        )
                        log.info(
                            "#"
                            + str(count)
                            + " Context Config ID: "
                            + str(taskjson1["ID"]),
                        )
                else:
                    payload = t1contextall[count]
                    url = url_link_final_2 + "api/contexts"
                    headers = {
                        "api-secret-key": tenant2key,
                        "api-version": "v1",
                        "Content-Type": "application/json",
                    }
                    response = requests.request(
                        "POST",
                        url,
                        headers=headers,
                        data=payload,
                        verify=cert,
                    )
                    describe = str(response.text)
                    taskjson = json.loads(describe)
                    t2contextid.append(str(taskjson["ID"]))
                    log.info(
                        "#" + str(count) + " Context Config name: " + taskjson["name"],
                    )
                    log.info(
                        "#" + str(count) + " Context Config ID: " + str(taskjson["ID"]),
                    )
            else:
                log.info(describe)
                log.info(payload)
    log.info("Done!")
    return t2contextid


def ScheduleGet(url_link_final, tenant1key):
    t1scheduleall = []
    t1schedulename = []
    t1scheduleid = []
    log.info("Getting All Schedule Configuration...")
    payload = {}
    url = url_link_final + "api/schedules"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request(
        "GET",
        url,
        headers=headers,
        data=payload,
        verify=cert,
    )
    describe = str(response.text)
    schedules_json = json.loads(describe).get("schedules")
    if schedules_json is not None:
        for count, here in enumerate(schedules_json):
            t1scheduleall.append(str(json.dumps(here)))
            t1schedulename.append(str(here["name"]))
            log.info(
                "#" + str(count) + " Schedule Config name: " + str(here["name"]),
            )
            t1scheduleid.append(str(here["ID"]))
            log.info("#" + str(count) + " Schedule Config ID: " + str(here["ID"]))
        log.info("Done")
    return t1scheduleall, t1schedulename, t1scheduleid


def ScheduleCreate(t1scheduleall, t1schedulename, url_link_final_2, tenant2key):
    schedule_id_dict = {}
    t2scheduleid = []
    log.info("Transfering All Schedule Configuration...")
    if t1schedulename:
        for count, schedule in enumerate(t1scheduleall):
            schedule_json = json.loads(schedule)
            old_id = schedule_json.get("ID")
            old_name = schedule_json.get("name")
            # for count, dirlist in enumerate(t1schedulename):
            payload = (
                '{"searchCriteria": [{"fieldName": "name","stringValue": "'
                + old_name
                + '"}]}'
            )
            url = url_link_final_2 + "api/schedules/search"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            taskjson = json.loads(describe)
            if not "message" in taskjson:
                if taskjson["schedules"]:
                    for here in taskjson["schedules"]:
                        indexid = here["ID"]
                        payload = t1scheduleall[count]
                        url = url_link_final_2 + "api/schedules/" + str(indexid)
                        headers = {
                            "api-secret-key": tenant2key,
                            "api-version": "v1",
                            "Content-Type": "application/json",
                        }
                        response = requests.request(
                            "POST",
                            url,
                            headers=headers,
                            data=payload,
                            verify=cert,
                        )
                        describe = str(response.text)
                        taskjson1 = json.loads(describe)
                        new_id = taskjson1["ID"]
                        t2scheduleid.append(str(taskjson1["ID"]))
                        log.info(
                            "#"
                            + str(count)
                            + " Schedule Config name: "
                            + taskjson1["name"],
                        )
                        log.info(
                            "#"
                            + str(count)
                            + " Schedule Config ID: "
                            + str(taskjson1["ID"]),
                        )
                else:
                    payload = t1scheduleall[count]
                    url = url_link_final_2 + "api/schedules"
                    headers = {
                        "api-secret-key": tenant2key,
                        "api-version": "v1",
                        "Content-Type": "application/json",
                    }
                    response = requests.request(
                        "POST",
                        url,
                        headers=headers,
                        data=payload,
                        verify=cert,
                    )
                    describe = str(response.text)
                    taskjson = json.loads(describe)
                    new_id = taskjson["ID"]
                    t2scheduleid.append(str(taskjson["ID"]))
                    log.info(
                        "#" + str(count) + " Schedule Config name: " + taskjson["name"],
                    )
                    log.info(
                        "#"
                        + str(count)
                        + " Schedule Config ID: "
                        + str(taskjson["ID"]),
                    )
                schedule_id_dict[old_id] = new_id
            else:
                log.info(describe)
                log.info(payload)
    log.info("Done!")
    return schedule_id_dict, t2scheduleid



import json
import os
import re
import sys
from typing import Union

import deepsecurity
import requests
from deepsecurity.rest import ApiException

from dsmigrator.logging import log
from dsmigrator.migrator_utils import safe_request


def to_snake(camel_case):
    pattern = re.compile(r"(?<!^)(?=[A-Z])")
    snake = pattern.sub("_", camel_case).lower()
    snake = snake.replace("application_type_i_d", "application_type_id")
    snake = snake.replace("_i_d", "_id")
    return snake


def CheckAPIAccess(url: str, tenantkey: str, cert: Union[str, bool] = False) -> bool:
    """
    Checks to make sure the API access key provided is valid for the given DSM URL

    Args:
        url (str): The url of the old DSM
        tenantkey (str): The API key to be checked
        cert (Union[str, bool], optional): Either the string of the cert file, or a boolean to disable cert checking. Defaults to False.

    Returns:
        bool: True if validation passes, False if it fails
    """
    url = url + "api/apikeys/current"
    result = None
    key_list = [i.split("-") for i in tenantkey.split(":")]
    first_identifier = key_list[0][0]
    key_list[0] = ["*" * len(i) for i in key_list[0][1:]]
    key_list[0].insert(0, first_identifier)
    key_list[1:] = [["*" * len(x) for x in i] for i in key_list[1:]]
    obfuscated_key = ":".join(["-".join(i) for i in key_list])
    try:
        payload = {}
        headers = {
            "api-secret-key": tenantkey,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "GET", url, headers=headers, data=payload, verify=cert
        )
        log.info(f"Checking api key {obfuscated_key}...")
        if "active" and "true" in response.text:
            result = True
        else:
            log.error(response.text)
            result = False
            log.error(
                "Double-check that your api key is correct, active, and has 'Full Access' permissions."
            )
            log.error("Aborting...")
            sys.exit(0)
    except Exception as e:
        log.exception(e)
        result = False
        log.error(
            "Double-check that your api key is correct, active, and has 'Full Access' permissions."
        )
        log.error("Aborting...")
        sys.exit(0)
    return result


class RestApiConfiguration:
    def __init__(
        self, NEW_API_KEY, NEW_HOST="https://cloudone.trendmicro.com", overrides=False
    ):
        self.configuration = deepsecurity.Configuration()
        self.api_client = deepsecurity.ApiClient(self.configuration)
        self.overrides = overrides
        self.configuration.host = "https://cloudone.trendmicro.com/api"
        self.configuration.api_key["api-secret-key"] = NEW_API_KEY
        self.api_version = "v1"

    def name_search_filter(self, name):
        criteria = deepsecurity.SearchCriteria()
        criteria.field_name = "name"
        criteria.string_test = "equal"
        criteria.string_value = f"%{name}%"
        return deepsecurity.SearchFilter(None, [criteria])


class SystemSettingsApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.SystemSettingsApi(self.api_client)

    def modify(self, json_settings):
        settings = deepsecurity.SystemSettings()
        for key in json_settings:
            setattr(settings, to_snake(key), json_settings[key])
        self.api_instance.modify_system_settings(settings, self.api_version)
        return settings


class ComputerGroupsApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.ComputerGroupsApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_computer_groups("v1", search_filter=filter)
        if results.computer_groups:
            return results.computer_groups[0].id

    def create(self, json_group):
        group = deepsecurity.ComputerGroup()
        for key in json_group:
            if not key == "ID":
                setattr(group, to_snake(key), json_group[key])
        self.api_instance.create_computer_group(group, self.api_version)
        return group.name


class EventBasedTasksApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.EventBasedTasksApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_event_based_tasks("v1", search_filter=filter)
        if results.event_based_tasks:
            return results.event_based_tasks[0].id

    def create(self, json_task):
        task = deepsecurity.EventBasedTask()
        for key in json_task:
            if not key == "ID":
                setattr(task, to_snake(key), json_task[key])
        self.api_instance.create_event_based_task(task, self.api_version)
        return task.name


class ScheduledTasksApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.ScheduledTasksApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_scheduled_tasks("v1", search_filter=filter)
        if results.scheduled_tasks:
            return results.scheduled_tasks[0].id

    def create(self, json_task):
        task = deepsecurity.ScheduledTask()
        for key in json_task:
            if not key == "ID":
                setattr(task, to_snake(key), json_task[key])
        self.api_instance.create_scheduled_task(task, self.api_version)
        return task.name


class DirectoryListsApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.DirectoryListsApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_directory_lists("v1", search_filter=filter)
        if results.directory_lists:
            return results.directory_lists[0].id

    def create(self, json_dirlist):
        dirlist = deepsecurity.DirectoryList()
        for key in json_dirlist:
            if not key == "ID":
                setattr(dirlist, to_snake(key), json_dirlist[key])
        self.api_instance.create_directory_list(dirlist, self.api_version)
        return dirlist.name


class FileExtensionListsApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.FileExtensionListsApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_file_extension_lists(
            "v1", search_filter=filter
        )
        if results.file_extension_lists:
            return results.file_extension_lists[0].id

    def create(self, json_extlist):
        extlist = deepsecurity.FileExtensionList()
        for key in json_extlist:
            if not key == "ID":
                setattr(extlist, to_snake(key), json_extlist[key])
        self.api_instance.create_file_extension_list(extlist, self.api_version)
        return extlist.name


class FileListsApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.FileListsApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_file_lists("v1", search_filter=filter)
        if results.file_lists:
            return results.file_lists[0].id

    def create(self, json_filelist):
        filelist = deepsecurity.FileList()
        for key in json_filelist:
            if not key == "ID":
                setattr(filelist, to_snake(key), json_filelist[key])
        self.api_instance.create_file_list(filelist, self.api_version)
        return filelist.name


class ApplicationTypesApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.ApplicationTypesApi(self.api_client)

    # def get(self, id):
    #     return self.api_instance.describe_application_type(id, "v1")

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_application_types("v1", search_filter=filter)
        if results.application_types:
            return results.application_types[0].id

    def create(self, json_application_types_list):
        application_type = deepsecurity.ApplicationType()
        for key in json_application_types_list:
            if not key == "ID":
                setattr(
                    application_type,
                    to_snake(key),
                    json_application_types_list[key],
                )
        self.api_instance.create_application_type(application_type, self.api_version)
        return application_type.name


class AMConfigApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.AntiMalwareConfigurationsApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_anti_malwares("v1", search_filter=filter)
        if results.anti_malware_configurations:
            return results.anti_malware_configurations[0].id

    def create(self, json):
        list = deepsecurity.AntiMalwareConfiguration()
        for key in json:
            if not key == "ID":
                setattr(list, to_snake(key), json[key])
        self.api_instance.create_anti_malware(list, self.api_version)
        return list.name


class FirewallApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.FirewallRulesApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_firewall_rules("v1", search_filter=filter)
        if results.firewall_rules:
            return results.firewall_rules[0].id

    def create(self, json):
        object = deepsecurity.FirewallRule()
        for key in json:
            if not key == "ID":
                setattr(object, to_snake(key), json[key])
        self.api_instance.create_firewall_rule(object, self.api_version)
        return object.name


class PolicyApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.PoliciesApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_policies("v1", search_filter=filter)
        if results.policies:
            return results.policies[0]

    def create(self, json):
        policy = deepsecurity.Policy()
        for key in json:
            if not key == "ID":
                setattr(policy, to_snake(key), json[key])
        self.api_instance.create_policy(policy, self.api_version)
        return policy.name

    def modify_parent(self, id, newparentid):
        policy = deepsecurity.Policy()
        policy.parent_id = newparentid
        self.api_instance.modify_policy(
            id, policy, self.api_version, overrides=self.overrides
        )

    def modify_real_time_id(self, id, newid):
        policy = deepsecurity.Policy()
        policy.anti_malware.real_time_scan_configuration_id = newid
        self.api_instance.modify_policy(
            id, policy, self.api_version, overrides=self.overrides
        )


class LogInspectionApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.LogInspectionRulesApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_log_inspection_rules(
            "v1", search_filter=filter
        )
        if results.log_inspection_rules:
            return results.log_inspection_rules[0].id

    def create(self, json):
        object = deepsecurity.LogInspectionRule()
        for key in json:
            if not key == "ID":
                setattr(object, to_snake(key), json[key])
        self.api_instance.create_log_inspection_rule(object, self.api_version)
        return object.name


class IntrusionPreventionApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.IntrusionPreventionRulesApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_intrusion_prevention_rules(
            "v1", search_filter=filter
        )
        if results.intrusion_prevention_rules:
            return results.intrusion_prevention_rules[0].id

    def create(self, json):
        object = deepsecurity.IntrusionPreventionRule()
        for key in json:
            if not key == "ID":
                setattr(object, to_snake(key), json[key])
        self.api_instance.create_intrusion_prevention_rule(object, self.api_version)
        return object.name


class IntegrityMonitoringApiInstance(RestApiConfiguration):
    def __init__(self, NEW_API_KEY, overrides=False):
        RestApiConfiguration.__init__(self, NEW_API_KEY, overrides)
        self.api_instance = deepsecurity.IntegrityMonitoringRulesApi(self.api_client)

    def search(self, name):
        filter = self.name_search_filter(name)
        results = self.api_instance.search_integrity_monitoring_rules(
            "v1", search_filter=filter
        )
        if results.integrity_monitoring_rules:
            return results.integrity_monitoring_rules[0].id

    def create(self, json):
        object = deepsecurity.IntegrityMonitoringRule()
        for key in json:
            if not key == "ID":
                setattr(object, to_snake(key), json[key])
        self.api_instance.create_integrity_monitoring_rule(
            self.api_version, integrity_monitoring_rule=object
        )
        return object.name



import json
import os
import sys
import time
from time import sleep

import requests
import urllib3
from deepsecurity.rest import ApiException
from rich.console import Console

from dsmigrator.api_config import AMConfigApiInstance
from dsmigrator.logging import log
from dsmigrator.migrator_utils import (
    safe_list_get,
    safe_request,
    validate_create,
    validate_create_dict,
)

cert = False


def json_to_list(json_item, key, outlist):
    item = json_item.get(key)
    if item:
        outlist.append(str(item))
        return outlist


def am_config_transform(antimalwareconfig, OLD_HOST, OLD_API_KEY):
    amdirectorylist, amfileextentionlist, amfilelist, allamconfig = AMconfigtenant1(
        antimalwareconfig, OLD_HOST, OLD_API_KEY
    )
    return amdirectorylist, amfileextentionlist, amfilelist, allamconfig


def am_validate_create(
    allofpolicy,
    antimalwareconfig,
    allamconfig,
    amdirectorylist,
    amalldirectorynew,
    amfileextentionlist,
    amallfileextentionnew,
    amfilelist,
    amallfilelistnew,
    schedule_id_dict,
    NEW_HOST,
    NEW_API_KEY,
):
    mod_allamconfig = AmConfigCheck(
        allamconfig,
        amdirectorylist,
        amalldirectorynew,
        amfileextentionlist,
        amallfileextentionnew,
        amfilelist,
        amallfilelistnew,
    )

    allamconfigdict = AmconfigTenant2(mod_allamconfig, NEW_API_KEY)

    aop_replace_am_configs = AmReplaceConfig(
        allofpolicy, allamconfigdict, schedule_id_dict
    )
    final = aop_replace_am_configs
    return final


def AMconfigtenant1(antimalwareconfig, url_link_final, tenant1key):
    allamconfig = []
    directorylist = []
    fileextentionlist = []
    filelist = []
    log.info("Getting Anti-Malware configuration from Tenant 1")
    for count, amconfig in enumerate(antimalwareconfig):
        if int(amconfig) != 0:
            url = url_link_final + "api/antimalwareconfigurations/" + str(amconfig)
            response = safe_request(tenant1key, "GET", url, payload={}, cert=cert)
            describe = str(response.text)
            allamconfig.append(describe)
            describe_json = json.loads(describe)
            json_to_list(describe_json, "directoryListID", directorylist)
            json_to_list(describe_json, "excludedDirectoryListID", directorylist)
            json_to_list(
                describe_json, "excludedFileExtensionListID", fileextentionlist
            )
            json_to_list(describe_json, "fileExtensionListID", fileextentionlist)
            json_to_list(describe_json, "excludedFileListID", filelist)
            json_to_list(describe_json, "fileListID", filelist)
            json_to_list(describe_json, "excludedProcessImageFileListID", filelist)
            log.info(
                "#" + str(count) + " Anti-Malware Config ID: " + str(amconfig),
            )
    directorylist = list(dict.fromkeys(directorylist))
    fileextentionlist = list(dict.fromkeys(fileextentionlist))
    filelist = list(dict.fromkeys(filelist))
    return directorylist, fileextentionlist, filelist, allamconfig


def AmconfigTenant2(allamconfig, NEW_API_KEY):
    log.info("Creating Anti-Malware Configuration in Tenant2")
    if allamconfig:
        allamconfigdict = validate_create_dict(
            allamconfig, AMConfigApiInstance(NEW_API_KEY), "anti malware"
        )
    log.info("New AM Config Dict")
    log.info(allamconfigdict)
    allamconfigdict[0] = 0
    return allamconfigdict


def AmReplaceConfig(allofpolicy, allamconfigdict, schedule_id_dict):
    count = 0
    for describe in allofpolicy:

        def replace_am_id(
            type,
            policy_json_string,
            newiddict,
        ):
            policy_json = json.loads(policy_json_string)
            policy_am_json = policy_json["antiMalware"]
            oldid = policy_am_json[type]
            policy_am_json[type] = newiddict[oldid]
            final_json = json.dumps(policy_json)
            return final_json

        describe_rt = replace_am_id(
            "realTimeScanConfigurationID", describe, allamconfigdict
        )
        describe_man = replace_am_id(
            "manualScanConfigurationID", describe_rt, allamconfigdict
        )
        describe_sch = replace_am_id(
            "scheduledScanConfigurationID", describe_man, allamconfigdict
        )
        describe_rt_sch = replace_am_id(
            "realTimeScanScheduleID", describe_sch, schedule_id_dict
        )
        allofpolicy[count] = describe_rt_sch
        count = count + 1
    return allofpolicy


def AmConfigCheck(
    allamconfig,
    directorylist,
    alldirectorynew,
    fileextentionlist,
    allfileextentionnew,
    filelist,
    allfilelistnew,
):

    new_allamconfig = []
    dir_list_dict = {
        int(directorylist[i]): int(safe_list_get(alldirectorynew, i))
        for i in range(len(directorylist))
    }
    file_list_dict = {
        int(filelist[i]): int(safe_list_get(allfilelistnew, i))
        for i in range(len(filelist))
    }
    file_ext_dict = {
        int(fileextentionlist[i]): int(safe_list_get(allfileextentionnew, i))
        for i in range(len(fileextentionlist))
    }
    for amconfig in allamconfig:

        def id_replacer(json: dict, rosetta: dict, key: str) -> dict:
            oldid = json.get(key)
            newid = rosetta.get(oldid)
            if newid is not None:
                json[key] = newid
            return json

        amconfig_json = json.loads(amconfig)

        amconfig_json = id_replacer(amconfig_json, dir_list_dict, "directoryListID")
        amconfig_json = id_replacer(
            amconfig_json, dir_list_dict, "excludedDirectoryListID"
        )
        amconfig_json = id_replacer(amconfig_json, file_ext_dict, "fileExtensionListID")
        amconfig_json = id_replacer(
            amconfig_json, file_ext_dict, "excludedFileExtensionListID"
        )
        amconfig_json = id_replacer(amconfig_json, file_list_dict, "excludedFileListID")
        amconfig_json = id_replacer(
            amconfig_json, file_list_dict, "excludedProcessImageFileListID"
        )
        new_amconfig = json.dumps(amconfig_json)
        new_allamconfig.append(new_amconfig)
    return new_allamconfig


def RenameAmConfig(allamconfig):
    count = 0
    if allamconfig:
        for describe in allamconfig:
            amjson = json.loads(describe)
            log.info(describe)
            amjson["name"] = amjson["name"] + " - Migrated"
            allamconfig[count] = json.dumps(amjson)
            count = count + 1
    return allamconfig



import sys
import os
import time
from time import sleep
import requests
import urllib3
import urllib3
import json
from dsmigrator.logging import log

cert = False


def li_config_transform(allofpolicy, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    liruleid = LIGet(allofpolicy)

    alllirule, allliruleidnew1, allliruleidold, alllicustomrule = LIDescribe(
        liruleid, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )

    allliruleidnew2 = LICustom(alllirule, alllicustomrule, NEW_HOST, NEW_API_KEY)

    aop_replace_li_rules = LIReplace(
        allofpolicy,
        allliruleidnew1,
        allliruleidnew2,
        liruleid,
        allliruleidold,
        alllicustomrule,
    )
    final = aop_replace_li_rules
    return final


def LIGet(allofpolicy):
    liruleid = []
    log.info("Log Inspection rules in Tenant 1")
    for describe in allofpolicy:
        namejson = json.loads(describe)
        if "ruleIDs" in namejson["logInspection"]:
            for count, here2 in enumerate(namejson["logInspection"]["ruleIDs"]):
                liruleid.append(str(here2))
    liruleid = list(dict.fromkeys(liruleid))
    log.info(liruleid)
    return liruleid


def LIDescribe(liruleid, url_link_final, tenant1key, url_link_final_2, tenant2key):
    alllirule = []
    alllirulename = []
    allliruleidnew1 = []
    allliruleidold = []
    alllicustomrule = []
    log.info("Searching LI rules in Tenant 1...")
    if liruleid:
        for count, dirlist in enumerate(liruleid):
            payload = {}
            url = url_link_final + "api/loginspectionrules/" + str(dirlist)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            alllirule.append(describe)
            lijson = json.loads(describe)
            alllirulename.append(str(lijson["name"]))
            log.info("#" + str(count) + " LI rule name: " + str(lijson["name"]))
            log.info("#" + str(count) + " LI rule ID: " + dirlist)
    log.info("Done!")
    log.info("Searching and Modifying LI rule in Tenant 2...")
    for count, dirlist in enumerate(alllirulename):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/loginspectionrules/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        taskjson = json.loads(describe)
        if not "message" in taskjson:
            index = describe.find(dirlist)
            if index != -1:
                index = describe.find('"ID"')
                if index != -1:
                    indexpart = describe[index + 4 :]
                    startIndex = indexpart.find(":")
                    if startIndex != -1:  # i.e. if the first quote was found
                        endIndex = indexpart.find("}", startIndex + 1)
                        if (
                            startIndex != -1 and endIndex != -1
                        ):  # i.e. both quotes were found
                            indexid = indexpart[startIndex + 1 : endIndex]
                            allliruleidnew1.append(str(indexid))
                            allliruleidold.append(count)
                            log.info("#" + str(count) + " LI rule ID: " + indexid)
                else:
                    log.info(describe)
                    log.info(payload)
            else:
                alllicustomrule.append(count)
        else:
            log.info(describe)
            log.info(payload)
    return alllirule, allliruleidnew1, allliruleidold, alllicustomrule


def LICustom(alllirule, alllicustomrule, url_link_final_2, tenant2key):
    allliruleidnew2 = []
    if alllicustomrule:
        log.info("Creating new custom LI rule in Tenant 2...")
        for count, indexnum in enumerate(alllicustomrule):
            payload = alllirule[indexnum]
            url = url_link_final_2 + "api/loginspectionrules"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        allliruleidnew2.append(str(indexid))
                        log.info("#" + str(count) + " LI rule ID: " + indexid)
            else:
                log.info(describe)
                log.info(payload)
        # log.info("all new LI rule custom rule")
        # log.info(allliruleidnew2)
        log.info("Done!")
    return allliruleidnew2


def LIReplace(
    allofpolicy,
    allliruleidnew1,
    allliruleidnew2,
    liruleid,
    allliruleidold,
    alllicustomrule,
):
    for count, describe in enumerate(allofpolicy):
        index = describe.find('"logInspection"')
        if index != -1:
            indexpart = describe[index + 14 :]
            startIndex = indexpart.find("}")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find("}", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    index2 = indexid.find("ruleIDs")
                    if index2 != -1:
                        indexpart2 = indexid[index2 + 9 :]
                        startIndex2 = indexpart2.find("[")
                        if startIndex2 != -1:  # i.e. if the first quote was found
                            endIndex2 = indexpart2.find("]", startIndex2 + 1)
                            if (
                                startIndex2 != -1 and endIndex2 != -1
                            ):  # i.e. both quotes were found
                                indexid2 = indexpart2[startIndex2 + 1 : endIndex2]
                                indexid3 = indexpart2[startIndex2 + 1 : endIndex2]
                                indexid4 = indexid2.split(", ")
                                if allliruleidnew1 or allliruleidnew2:
                                    for count1, this in enumerate(indexid4):
                                        checkindex = liruleid.index(this)
                                        if checkindex in allliruleidold:
                                            checkindex1 = allliruleidold.index(
                                                checkindex
                                            )
                                            indexid4[count1] = allliruleidnew1[
                                                checkindex1
                                            ]
                                        elif checkindex in alllicustomrule:
                                            checkindex1 = alllicustomrule.index(
                                                checkindex
                                            )
                                            indexid4[count1] = allliruleidnew2[
                                                checkindex1
                                            ]
                                    indexid2 = ",".join(indexid4)
                                modulepart = describe[index : index + 14 + endIndex]
                                modulepart2 = modulepart.replace(indexid3, indexid2)
                                allofpolicy[count] = describe.replace(
                                    modulepart, modulepart2
                                )
    return allofpolicy



import sys
import os
import requests
import urllib3
import urllib3
import json
from dsmigrator.logging import log
from nested_lookup import nested_lookup, nested_update
from dsmigrator.api_config import ScheduledTasksApiInstance, EventBasedTasksApiInstance
from dsmigrator.migrator_utils import validate_create, rename_json

cert = False


def ebt_listmaker(
    policy_dict, computer_group_dict, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
):
    enum_oldetname, etIDs = ListEventTask(OLD_HOST, OLD_API_KEY)
    if etIDs:
        allet, nameet = GetEventTask(etIDs, OLD_HOST, OLD_API_KEY)
        CreateEventTask(
            allet, nameet, policy_dict, computer_group_dict, NEW_HOST, NEW_API_KEY
        )


def st_listmaker(
    policy_dict, computer_group_dict, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
):
    enum_oldstname, stIDs = ListScheduledTask(OLD_HOST, OLD_API_KEY)
    if stIDs:
        allst, namest = GetScheduledTask(stIDs, OLD_HOST, OLD_API_KEY)
        CreateScheduledTask(
            allst, namest, policy_dict, computer_group_dict, NEW_HOST, NEW_API_KEY
        )


def ListEventTask(url_link_final, tenant1key):
    payload = {}
    url = url_link_final + "api/eventbasedtasks"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request(
        "GET",
        url,
        headers=headers,
        data=payload,
        verify=cert,
    )
    describe = str(response.text)
    oldetname = []
    oldetid = []
    ebt_json = json.loads(describe).get("eventBasedTasks")
    if ebt_json is not None:
        for here in ebt_json:
            oldetname.append(str(here["name"]))
            oldetid.append(str(here["ID"]))
    return enumerate(oldetname), oldetid


def GetEventTask(etIDs, url_link_final, tenant1key):
    allet = []
    nameet = []
    log.info("Getting Target Task...")
    if etIDs:
        for part in etIDs:
            payload = {}
            url = url_link_final + "api/eventbasedtasks/" + str(part)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            allet.append(describe)
            namejson = json.loads(describe)
            nameet.append(str(namejson["name"]))
        return allet, nameet


def CreateEventTask(
    allet, nameet, policy_dict, computer_group_dict, url_link_final_2, tenant2key
):
    log.info("Creating Task to target Account...")
    if nameet:
        modet = []
        for task in allet:
            task_json = rename_json(json.loads(task))
            oldname = task_json.get("name")
            actions_list = task_json.get("actions")
            if actions_list is not None:
                for action in actions_list:
                    oldid = action.get("parameterValue")
                    if oldid is not None:
                        if (computer_group_dict.get(oldid) is not None) and (
                            action["type"] == "assign-group"
                        ):
                            action["parameterValue"] = computer_group_dict[oldid]
                        elif computer_group_dict.get(oldid) is None:
                            log.info(
                                f"WARNING: THE COMPUTER GROUP ASSIGNED IN {oldname} DOES NOT EXIST. TASK WILL NOT BE MIGRATED"
                            )
                            task_json = {}
                        if (policy_dict.get(oldid) is not None) and (
                            action["type"] == "assign-policy"
                        ):
                            action["parameterValue"] = policy_dict[oldid]
                        elif policy_dict.get(oldid) is None:
                            log.info(
                                f"WARNING: THE POLICY ASSIGNED IN {oldname} DOES NOT EXIST. TASK WILL NOT BE MIGRATED"
                            )
                            task_json = {}
            if task_json:
                modet.append(json.dumps(task_json))

        validate_create(modet, EventBasedTasksApiInstance(tenant2key), "Event Based")


def ListScheduledTask(url_link_final, tenant1key):
    payload = {}
    url = url_link_final + "api/scheduledtasks"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request(
        "GET",
        url,
        headers=headers,
        data=payload,
        verify=cert,
    )
    oldstname = []
    oldstid = []
    st_json = json.loads(str(response.text)).get("scheduledTasks")
    if st_json is not None:
        for here in st_json:
            oldstname.append(str(here["name"]))
            oldstid.append(str(here["ID"]))
    return enumerate(oldstname), oldstid


def GetScheduledTask(stIDs, url_link_final, tenant1key):
    allst = []
    namest = []
    log.info("Getting Target Task...")
    if stIDs:
        for part in stIDs:
            payload = {}
            url = url_link_final + "api/scheduledtasks/" + str(part)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            allst.append(describe)
            namejson = json.loads(describe)
            namest.append(str(namejson["name"]))
    return allst, namest


def CreateScheduledTask(
    allst, namest, policy_dict, computer_group_dict, url_link_final_2, tenant2key
):
    log.info("Creating Task to target Account...")
    if namest:
        modst = []
        for task in allst:
            task_json = rename_json(json.loads(task))
            oldname = task_json.get("name")
            oldcomputergroup = nested_lookup("computerGroupID", task_json)
            oldpolicy = nested_lookup("policyID", task_json)
            has_smartfolder = bool(nested_lookup("smartFolderID", task_json))
            if oldcomputergroup:
                oldid = oldcomputergroup[0]
                if computer_group_dict.get(oldid) is not None:
                    task_json = nested_update(
                        task_json,
                        "computerGroupID",
                        computer_group_dict[oldid],
                    )
                else:
                    log.info(
                        f"WARNING: THE COMPUTER GROUP ASSIGNED IN {oldname} DOES NOT EXIST. TASK WILL NOT BE MIGRATED"
                    )
                    task_json = {}
            if oldpolicy:
                oldid = oldpolicy[0]
                if computer_group_dict.get(oldid) is not None:
                    task_json = nested_update(task_json, "policyID", policy_dict[oldid])
                else:
                    log.info(
                        f"WARNING: THE POLICY ASSIGNED IN {oldname} DOES NOT EXIST. TASK WILL NOT BE MIGRATED"
                    )
                    task_json = {}
            if has_smartfolder:
                log.info(
                    f"WARNING: THE SMART FOLDER ASSIGNED IN {oldname} CANNOT BE MIGRATED."
                )
                task_json = {}
            if task_json:
                modst.append(json.dumps(task_json))

        validate_create(modst, ScheduledTasksApiInstance(tenant2key), "Scheduled")



import codecs
import datetime
import logging
import os
import sys
import time
import traceback
from datetime import datetime

import click
import deepsecurity
import requests
import urllib3
import yaml
from rich.console import Console, OverflowMethod
from rich.logging import RichHandler
from rich.traceback import install

from dsmigrator.antimalware import am_config_transform, am_validate_create
from dsmigrator.api_config import CheckAPIAccess
from dsmigrator.computer_groups import computer_group_listmaker
from dsmigrator.firewall import firewall_config_transform
from dsmigrator.integrity import im_config_transform
from dsmigrator.ips import ips_rules_transform
from dsmigrator.lists import (
    context_listmaker,
    directory_listmaker,
    ip_listmaker,
    mac_listmaker,
    port_listmaker,
    schedule_listmaker,
    stateful_listmaker,
)
from dsmigrator.logging import console, log
from dsmigrator.loginspection import li_config_transform
from dsmigrator.policies import (
    AddPolicy,
    GetPolicy,
    ListAllPolicy,
    delete_cloud_one_policies,
)
from dsmigrator.proxy import proxy_edit
from dsmigrator.tasks import ebt_listmaker, st_listmaker

# Welcome Banner
console.print(
    """\
                                                      
              %###################,               
          #############################           
       ######################\\.         `#        
     #################################,    \\      
   #####################################     #    
  #######################################     #   
 ##############  ########################    .##  
(##########(    ########################     ###( 
#####(              ###################     ##### 
######(      ########################     (###### 
######       #######################      #######
 ####       ######################      ######### 
  ###       ###################      ########### 
   #        ###############       (#############  
    #         #######.         ###############&   
      \\                   .#################     
        %.          ######################        
           \\###########################           
               ###################               
    """,
    style="bold red",
    overflow="crop",
)
console.print(
    "Welcome to the Trend Micro Policy Migration Tool",
    style="bold red",
)


# Init helpers


def validate_url(ctx, param, url: str) -> str:
    """
    Makes sure urls start with 'http' and end with a '/'

    Args:
        ctx: necessary for click
        param: necessary for click
        url (str): url string

    Raises:
        click.BadParameter: kicks the user out of the cli and lets them know
                            about the specific error

    Returns:
        str: a sanitized url
    """
    try:
        assert url[0:4] == "http"
        if url[-1] != "/":
            url = url + "/"
        return url
    except:
        raise click.BadParameter(
            "DSM url must start with http, please double-check input"
        )


def validate_api_keys(ctx, param, api_key):
    try:
        last_equal = api_key[-1] == "="
        first_validation_bit = api_key.split("-")
        last_validation_bit = first_validation_bit[4].split(":")
        clean_list = [i for i in first_validation_bit[0:4]]
        clean_list.append(last_validation_bit[0])
        validate_list = [len(i) for i in clean_list]
        validate_list.append(last_equal)

        if validate_list == [8, 4, 4, 4, 12, True]:
            return api_key
        else:
            raise click.BadParameter(
                "Invalid API key format, please double-check the input."
            )
    except:
        raise click.BadParameter(
            "Invalid API key format, please double-check the input."
        )


# override the click invoke method


def CommandWithConfigFile(config_file_param_name):
    class CustomCommandClass(click.Command):
        def invoke(self, ctx):
            config_file = ctx.params[config_file_param_name]
            if config_file is not None:
                with open(config_file) as f:
                    config_data = yaml.safe_load(f)
                    for param, value in ctx.params.items():
                        if value is None and param in config_data:
                            ctx.params[param] = config_data[param]
                            ctx.params[param].prompt = None

            return super(CustomCommandClass, self).invoke(ctx)

    return CustomCommandClass


# pass options


@click.command(cls=CommandWithConfigFile("config_file"))
@click.option("--config-file", type=click.Path(), is_eager=True)
@click.option(
    "-ou",
    "--original-url",
    callback=validate_url,
    prompt="Old DSM address and port (e.g. https://10.10.10.10:4119/)",
    help="A resolvable FQDN for the old DSM, with port number (e.g. https://192.168.1.1:4119/)",
    envvar="ORIGINAL_URL",
)
@click.option(
    "-oa",
    "--original-api-key",
    callback=validate_api_keys,
    prompt="Old DSM API key",
    help="API key for the old DSM with Full Access permissions",
    envvar="ORIGINAL_API_KEY",
)
@click.option(
    "-nu",
    "--new-url",
    default="https://cloudone.trendmicro.com/",
    show_default=True,
    help="Destination url",
)
@click.option(
    "-coa",
    "--cloud-one-api-key",
    prompt="New Cloud One API key",
    help="API key for Cloud One Workload Security with Full Access permissions",
    envvar="CLOUD_ONE_API_KEY",
)
@click.option(
    "-d",
    "--delete-policies/--keep-policies",
    is_flag=True,
    default=False,
    prompt="Do you want to wipe existing policies in Cloud One? (not required, but will give best results)",
    help="Wipes existing policies in Cloud One (not required, but will give best results)",
)
@click.option(
    "-t",
    "--tasks",
    is_flag=True,
    help="(BETA) Enable the task migrator (may be buggy)",
)
@click.option(
    "-k",
    "--insecure",
    is_flag=True,
    help="Suppress the InsecureRequestWarning for self-signed certificates",
)
@click.option(
    "-f",
    "--filter",
    help="A list of policy names in form '[name, name, ...]' which are the only ones which will be transferred.",
)
# @click.option(
#     "-c",
#     "--cert",
#     default=False,
#     show_default=True,
#     help="(Optional) Allows the use of a cert file",
# )

# Main tool


def main(
    config_file,
    original_url,
    original_api_key,
    new_url,
    cloud_one_api_key,
    # cert,
    insecure,
    tasks,
    delete_policies,
    filter,
):
    """Moves your on-prem DS deployment to the cloud!"""
    OLD_API_KEY = original_api_key
    OLD_HOST = original_url
    NEW_API_KEY = cloud_one_api_key
    NEW_HOST = new_url

    log.info("starting tool")

    if insecure:
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    # Double-check that creds will work
    console.rule("Validating Credentials")
    validation_result1 = CheckAPIAccess(original_url, original_api_key)
    validation_result2 = CheckAPIAccess(new_url, cloud_one_api_key)

    if validation_result1 and validation_result2:
        log.info("Successfully authenticated!")
    else:
        log.error("Something went wrong with authentication.")
        log.error(
            "Double-check that your api key is correct, active, and has 'Full Access' permissions."
        )
        log.error("Aborting...")
        sys.exit(0)

    if delete_policies:
        console.rule("Delete C1 Policies")
        delete_cloud_one_policies(NEW_API_KEY)

    # Populate Initial Data
    old_policy_id_list, oldpolicynameid_dict = ListAllPolicy(OLD_HOST, OLD_API_KEY)

    # Filter Module
    if filter:
        console.rule("Filter Out Unwanted Policies")
        if (filter[0] != "[") or (filter[-1] != "]"):
            log.error(
                "Please pass in filter names in form '[name1, name2, ...]', making note of brackets"
            )
            raise TypeError
        name_list = filter[1:-1].split(", ")
        old_policy_id_list = []
        for desired_policy in name_list:
            # validate
            if '"' in desired_policy:
                log.error(
                    "Please pass in filter names in form '[name1, name2, ...]', making note of quoting conventions"
                )
                raise TypeError
            desired_id = oldpolicynameid_dict.get(desired_policy)
            if desired_id is not None:
                old_policy_id_list.append(desired_id)
        log.info(f"New desired policy IDs: {old_policy_id_list}")

    console.rule("Initial Data Collection")

    antimalwareconfig, allofpolicy = GetPolicy(
        old_policy_id_list, OLD_HOST, OLD_API_KEY
    )

    console.rule("Anti-Malware Configurations")
    amdirectorylist, amfileextensionlist, amfilelist, allamconfig = am_config_transform(
        antimalwareconfig, OLD_HOST, OLD_API_KEY
    )

    console.rule("Lists")
    amalldirectorynew, amallfileextentionnew, amallfilelistnew = directory_listmaker(
        amdirectorylist,
        amfileextensionlist,
        amfilelist,
        OLD_HOST,
        OLD_API_KEY,
        NEW_API_KEY,
    )

    t1portlistid, t2portlistid = port_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    t1maclistid, t2maclistid = mac_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    t1iplistid, t2iplistid = ip_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY)
    (
        t1statefulid,
        t2statefulid,
        stateful_dict,
    ) = stateful_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY)
    t1contextid, t2contextid = context_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    schedule_id_dict, t1scheduleid, t2scheduleid = schedule_listmaker(
        OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )

    console.rule("Proxy Settings")
    try:
        proxy_edit(allofpolicy, t1iplistid, t2iplistid, t1portlistid, t2portlistid)
    except Exception as e:
        log.exception(e)
        log.error("There was a critical error in transferring proxy settings.")
        log.error(
            "Transfer will continue, but please double check the proxy settings in Cloud One"
        )
        pass

    # TRANSFORM
    console.rule("Intrusion Prevention Module")
    allofpolicy = ips_rules_transform(
        allofpolicy,
        t1portlistid,
        t2portlistid,
        t1scheduleid,
        t2scheduleid,
        t1contextid,
        t2contextid,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )
    console.rule("Anti-Malware Module")
    allofpolicy = am_validate_create(
        allofpolicy,
        antimalwareconfig,
        allamconfig,
        amdirectorylist,
        amalldirectorynew,
        amfileextensionlist,
        amallfileextentionnew,
        amfilelist,
        amallfilelistnew,
        schedule_id_dict,
        NEW_HOST,
        NEW_API_KEY,
    )
    console.rule("Integrity Monitoring Module")
    allofpolicy = im_config_transform(
        allofpolicy, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    console.rule("Log Inspection Module")
    allofpolicy = li_config_transform(
        allofpolicy, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )
    console.rule("Firewall Module")
    allofpolicy = firewall_config_transform(
        allofpolicy,
        t1iplistid,
        t2iplistid,
        t1maclistid,
        t2maclistid,
        t1portlistid,
        t2portlistid,
        t1statefulid,
        t2statefulid,
        t1scheduleid,
        t2scheduleid,
        t1contextid,
        t2contextid,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )
    console.rule("Final Policy Migration")
    policy_dict = AddPolicy(allofpolicy, NEW_API_KEY)
    if tasks:
        console.rule("Tasks")
        computer_group_dict = computer_group_listmaker(
            OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
        )
        ebt_listmaker(
            policy_dict,
            computer_group_dict,
            OLD_HOST,
            OLD_API_KEY,
            NEW_HOST,
            NEW_API_KEY,
        )
        st_listmaker(
            policy_dict,
            computer_group_dict,
            OLD_HOST,
            OLD_API_KEY,
            NEW_HOST,
            NEW_API_KEY,
        )


if __name__ == "__main__":
    try:
        main()  # pylint: disable=no-value-for-parameter
    except Exception:
        log.exception("Fatal error in main.")



import json
from typing import Dict, List

import requests
import urllib3
from deepsecurity.rest import ApiException
from nested_lookup import nested_lookup, nested_update

from dsmigrator.api_config import (
    ApplicationTypesApiInstance,
    IntrusionPreventionApiInstance,
)

from dsmigrator.logging import log

from dsmigrator.migrator_utils import (
    validate_create,
    validate_create_dict,
    validate_create_dict_custom,
)

cert = False


def ips_rules_transform(
    allofpolicy,
    t1portlistid,
    t2portlistid,
    t1scheduleid,
    t2scheduleid,
    t1contextid,
    t2contextid,
    OLD_HOST,
    OLD_API_KEY,
    NEW_HOST,
    NEW_API_KEY,
):
    og_ipsruleid_dict = IPSGet(allofpolicy)
    og_ipsappid_dict = IPSappGet(allofpolicy)

    ipsappid_dict, ipscustomapp_dict = IPSappDescribe(
        og_ipsappid_dict,
        t1portlistid,
        t2portlistid,
        OLD_HOST,
        NEW_HOST,
        OLD_API_KEY,
        NEW_API_KEY,
    )

    ipsruleid_dict, ipscustomrule_dict = IPSDescribe(
        og_ipsruleid_dict,
        t1scheduleid,
        t2scheduleid,
        t1contextid,
        t2contextid,
        ipsappid_dict,
        ipscustomapp_dict,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )

    aop_replace_ips_apps = IPSappReplace(allofpolicy, ipsappid_dict, ipscustomapp_dict)
    aop_replace_ips_rules = IPSReplace(
        aop_replace_ips_apps, ipsruleid_dict, ipscustomrule_dict
    )
    return aop_replace_ips_rules


def IPSappGet(allofpolicy: List[str]) -> Dict[int, None]:
    """
    Parses allofpolicy and returns an IPS app skeleton dict

    Args:
        allofpolicy (List[str]): allofpolicy list of json strings

    Returns:
        Dict[int, None]: ips app id dict, with oldids, ready to be matched with new ids
    """
    ipsappid = []
    for describe in allofpolicy:
        namejson = json.loads(describe)
        if "applicationTypeIDs" in namejson["intrusionPrevention"]:
            for assigned_app_id in namejson["intrusionPrevention"][
                "applicationTypeIDs"
            ]:
                ipsappid.append(assigned_app_id)
    ipsappid_dict = dict.fromkeys(ipsappid)
    log.info("IPS application types in Tenant 1:")
    log.info(ipsappid)
    return ipsappid_dict


def IPSappDescribe(
    ipsappid_dict,
    t1portlistid,
    t2portlistid,
    url_link_final,
    url_link_final_2,
    tenant1key,
    tenant2key,
):
    allipsapp = []
    allipsappname = []
    allipscustomapp = []
    ipsapp_api_instance = ApplicationTypesApiInstance(tenant2key)

    log.info("Searching IPS application types in Tenant 1...")
    if ipsappid_dict:
        for count, name in enumerate(list(ipsappid_dict.keys())):
            payload = {}
            url = url_link_final + "api/applicationtypes/" + str(name)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            try:
                ipsappjson = json.loads(describe)
                allipsappname.append(str(ipsappjson["name"]))
                log.info(
                    "#"
                    + str(count)
                    + " IPS Application Type name: "
                    + str(ipsappjson["name"]),
                )
                old_port_list_id = ipsappjson.get("portListID")
                if old_port_list_id is not None:
                    indexnum = t1portlistid.index(str(old_port_list_id))
                    ipsappjson["portListID"] = t2portlistid[indexnum]
                allipsapp.append(json.dumps(ipsappjson))
                log.info(
                    "#"
                    + str(count)
                    + " IPS Application Type ID: "
                    + str(ipsappjson["ID"]),
                )
            except:

                log.info(describe)
    log.info("Done!")
    log.info("Searching and Modifying IPS application types in Tenant 2...")

    allipscustomapp = []
    for (count, object) in enumerate(allipsapp):
        namecheck = 1
        rename = 1
        object_json = json.loads(object)
        old_id = object_json["ID"]
        old_name = object_json["name"]
        while namecheck != -1:
            try:
                new_id = ipsapp_api_instance.search(old_name)
                if new_id is not None:
                    ipsappid_dict[old_id] = new_id
                    log.info(
                        f"#{str(count)} IPS Application Type: {old_name}",
                    )
                else:
                    allipscustomapp.append(json.dumps(object_json))
                namecheck = -1
            except ApiException as e:
                if "already exists" in e.body:
                    log.info(f"{old_name} already exists in new tenant, renaming...")
                    object_json["name"] = old_name + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    log.info(e.body)
                    pass
    log.info("Done!")
    if allipscustomapp:
        ipscustomapp_dict = validate_create_dict(
            allipscustomapp, ipsapp_api_instance, "IPS Custom App"
        )
    else:
        ipscustomapp_dict = {}
    log.info("Done!")
    return ipsappid_dict, ipscustomapp_dict


def IPSappReplace(allofpolicy, ipsappid_dict, ipscustomapp_dict):
    for count, policy in enumerate(allofpolicy):
        policyjson = json.loads(policy)
        if "applicationTypeIDs" in policyjson["intrusionPrevention"]:
            all_ipsapp_ids_list = policyjson["intrusionPrevention"][
                "applicationTypeIDs"
            ]
            for index, ipsapp_id in enumerate(all_ipsapp_ids_list):
                new_ipsapp_id = ipsappid_dict.get(ipsapp_id)
                new_ipscustomapp_id = ipscustomapp_dict.get(ipsapp_id)
                if new_ipsapp_id is not None:
                    all_ipsapp_ids_list[index] = new_ipsapp_id
                elif new_ipscustomapp_id is not None:
                    all_ipsapp_ids_list[index] = new_ipscustomapp_id
        allofpolicy[count] = json.dumps(policyjson)
    return allofpolicy


def IPSGet(allofpolicy):
    # Takes in allofpolicy and creates a skeleton id dict
    ipsruleid = []
    for describe in allofpolicy:
        namejson = json.loads(describe)
        if "ruleIDs" in namejson["intrusionPrevention"]:
            for assigned_rule_id in namejson["intrusionPrevention"]["ruleIDs"]:
                ipsruleid.append(assigned_rule_id)
    ipsruleid_dict = dict.fromkeys(ipsruleid)
    log.info("IPS rules in Tenant 1:")
    log.info(ipsruleid)
    return ipsruleid_dict


def IPSDescribe(
    ipsruleid_dict,
    t1scheduleid,
    t2scheduleid,
    t1contextid,
    t2contextid,
    ips_appid_dict,
    ipscustomapp_dict,
    url_link_final,
    tenant1key,
    url_link_final_2,
    tenant2key,
):
    allipsrule = []
    allipsrulename = []
    ipsrule_api_instance = IntrusionPreventionApiInstance(tenant2key)
    log.info("Searching IPS rules in Tenant 1...")

    if ipsruleid_dict:
        for count, dirlist in enumerate(list(ipsruleid_dict.keys())):
            payload = {}
            url = url_link_final + "api/intrusionpreventionrules/" + str(dirlist)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            try:
                ipsjson = json.loads(describe)
                allipsrulename.append(str(ipsjson["name"]))
                log.info(
                    "#" + str(count) + " IPS Rule name: " + str(ipsjson["name"]),
                )
                old_appid = ipsjson.get("applicationTypeID")
                new_premade_appid = ips_appid_dict.get(old_appid)
                new_custom_appid = ipscustomapp_dict.get(old_appid)
                if new_premade_appid is not None:
                    ipsjson["applicationTypeID"] = new_premade_appid
                elif new_custom_appid is not None:
                    ipsjson["applicationTypeID"] = new_custom_appid
                if "scheduleID" in ipsjson:
                    indexnum = t1scheduleid.index(str(ipsjson["scheduleID"]))
                    ipsjson["scheduleID"] = t2scheduleid[indexnum]
                if "contextID" in ipsjson:
                    indexnum = t1contextid.index(str(ipsjson["contextID"]))
                    ipsjson["contextID"] = t2contextid[indexnum]

                log.info("#" + str(count) + " IPS Rule ID: " + str(ipsjson["ID"]))
                allipsrule.append(json.dumps(ipsjson))
            except:
                log.info(describe)

    log.info("Done!")
    log.info("Searching and Modifying IPS rule in Tenant 2...")
    ipsruleid_dict, allipscustomrule = validate_create_dict_custom(
        allipsrule, ipsruleid_dict, ipsrule_api_instance, "IPS Rule"
    )
    if allipscustomrule:
        ipscustomrule_dict = validate_create_dict(
            allipscustomrule, ipsrule_api_instance, "IPS Custom Rule"
        )
    else:
        ipscustomrule_dict = {}
    log.info("Done!")
    return ipsruleid_dict, ipscustomrule_dict


def IPSReplace(
    allofpolicy: List[str],
    ipsruleid_dict: Dict[int, int],
    ipscustomrule_dict: Dict[int, int],
) -> List[str]:
    """
    Takes in the rule dictionaries for custom and predefined rules and transforms allofpolicy to match1

    Args:
        allofpolicy (List[str]): list of json strings representing policies
        ipsruleid_dict (Dict[int, int]): mapping of oldid:newid for ipsrules
        ipscustomrule_dict (Dict[int, int]): mapping of oldid:newid for ips custom rules

    Returns:
        List[str]: allofpolicy with ips rule ids modified
    """
    for count, policy in enumerate(allofpolicy):
        policyjson = json.loads(policy)
        if "ruleIDs" in policyjson["intrusionPrevention"]:
            all_ips_rule_ids_list = policyjson["intrusionPrevention"]["ruleIDs"]
            for index, ipsrule_id in enumerate(all_ips_rule_ids_list):
                new_ipsrule_id = ipsruleid_dict.get(ipsrule_id)
                new_ipscustomrule_id = ipscustomrule_dict.get(ipsrule_id)
                if new_ipsrule_id is not None:
                    all_ips_rule_ids_list[index] = new_ipsrule_id
                elif new_ipscustomrule_id is not None:
                    all_ips_rule_ids_list[index] = new_ipscustomrule_id
                else:
                    all_ips_rule_ids_list.remove(ipsrule_id)
                    log.warning(f"Removing ips rule {ipsrule_id} as outlined earlier.")
        allofpolicy[count] = json.dumps(policyjson)
    return allofpolicy



import sys
import os
import time
from time import sleep
import requests
import urllib3
import urllib3
import json
from dsmigrator.logging import log


def proxy_edit(allofpolicy, t1iplistid, t2iplistid, t1portlistid, t2portlistid):
    if allofpolicy:
        for count, describe in enumerate(allofpolicy):
            policyjson = json.loads(describe)
            if (
                "true"
                in policyjson["policySettings"][
                    "antiMalwareSettingSmartProtectionGlobalServerUseProxyEnabled"
                ]["value"]
            ):
                policyjson["policySettings"][
                    "antiMalwareSettingSmartProtectionGlobalServerUseProxyEnabled"
                ]["value"] = "false"

            if (
                "true"
                in policyjson["policySettings"][
                    "webReputationSettingSmartProtectionGlobalServerUseProxyEnabled"
                ]["value"]
            ):
                policyjson["policySettings"][
                    "webReputationSettingSmartProtectionGlobalServerUseProxyEnabled"
                ]["value"] = "false"

            if (
                "true"
                in policyjson["policySettings"][
                    "platformSettingSmartProtectionGlobalServerUseProxyEnabled"
                ]["value"]
            ):
                policyjson["policySettings"][
                    "platformSettingSmartProtectionGlobalServerUseProxyEnabled"
                ]["value"] = "false"

            policyjson["policySettings"][
                "platformSettingSmartProtectionAntiMalwareGlobalServerProxyId"
            ]["value"] = ""
            policyjson["policySettings"][
                "webReputationSettingSmartProtectionWebReputationGlobalServerProxyId"
            ]["value"] = ""
            policyjson["policySettings"][
                "platformSettingSmartProtectionGlobalServerProxyId"
            ]["value"] = ""

            here = policyjson["policySettings"][
                "firewallSettingReconnaissanceExcludeIpListId"
            ]["value"]
            if here:
                if here != "NONE":
                    indexnum = t1iplistid.index(here)
                    policyjson["policySettings"][
                        "firewallSettingReconnaissanceExcludeIpListId"
                    ]["value"] = t2iplistid[indexnum]

            here = policyjson["policySettings"][
                "firewallSettingReconnaissanceIncludeIpListId"
            ]["value"]
            if here:
                if here != "NONE":
                    indexnum = t1iplistid.index(here)
                    policyjson["policySettings"][
                        "firewallSettingReconnaissanceIncludeIpListId"
                    ]["value"] = t2iplistid[indexnum]

            here = policyjson["policySettings"][
                "firewallSettingEventLogFileIgnoreSourceIpListId"
            ]["value"]
            if here:
                if here != "NONE":
                    indexnum = t1iplistid.index(here)
                    try:
                        policyjson["policySettings"][
                            "firewallSettingEventLogFileIgnoreSourceIpListId"
                        ]["value"] = t2iplistid[indexnum]
                    except Exception as e:
                        log.exception(e)
                        log.error(
                            f"Error in policy settings transfer for firewall event exceptions for {policyjson.get('name')}. Transfer will continue, but double check after transfer."
                        )
                        pass

            here = policyjson["policySettings"][
                "webReputationSettingMonitorPortListId"
            ]["value"]
            if here:
                if here != "80,8080":
                    indexnum = t1portlistid.index(here)
                    policyjson["policySettings"][
                        "webReputationSettingMonitorPortListId"
                    ]["value"] = t2portlistid[indexnum]
            allofpolicy[count] = json.dumps(policyjson)
    return allofpolicy



import sys
import os
import time
from time import sleep
import requests
import urllib3
import urllib3
import json
from dsmigrator.logging import log

cert = False


def firewall_config_transform(
    allofpolicy,
    t1iplistid,
    t2iplistid,
    t1maclistid,
    t2maclistid,
    t1portlistid,
    t2portlistid,
    t1statefulid,
    t2statefulid,
    t1scheduleid,
    t2scheduleid,
    t1contextid,
    t2contextid,
    OLD_HOST,
    OLD_API_KEY,
    NEW_HOST,
    NEW_API_KEY,
):
    firewallruleid, policystateful = FirewallGet(allofpolicy)
    (
        allfirewallrule,
        allfirewallruleidnew1,
        allfirewallruleidold,
        allfirewallcustomrule,
    ) = FirewallDescribe(
        firewallruleid,
        t1iplistid,
        t2iplistid,
        t1maclistid,
        t2maclistid,
        t1portlistid,
        t2portlistid,
        t1scheduleid,
        t2scheduleid,
        t1contextid,
        t2contextid,
        OLD_HOST,
        OLD_API_KEY,
        NEW_HOST,
        NEW_API_KEY,
    )
    allfirewallruleidnew2 = FirewallCustom(
        allfirewallrule, allfirewallcustomrule, NEW_HOST, NEW_API_KEY
    )

    new_allofpolicy = FirewallReplace(
        allofpolicy,
        allfirewallruleidnew1,
        allfirewallruleidnew2,
        firewallruleid,
        allfirewallruleidold,
        allfirewallcustomrule,
        t1statefulid,
        t2statefulid,
    )
    return new_allofpolicy


def FirewallGet(allofpolicy):
    firewallruleid = []
    policystateful = []
    # find all Firewall rules
    log.info("Firewall rules in Tenant 1")
    for describe in allofpolicy:
        namejson = json.loads(describe)
        if "globalStatefulConfigurationID" in namejson["firewall"]:
            policystateful.append(
                str(namejson["firewall"]["globalStatefulConfigurationID"])
            )
        if "ruleIDs" in namejson["firewall"]:
            for count, here2 in enumerate(namejson["firewall"]["ruleIDs"]):
                firewallruleid.append(str(here2))
    firewallruleid = list(dict.fromkeys(firewallruleid))
    log.info(firewallruleid)
    return firewallruleid, policystateful


def FirewallDescribe(
    firewallruleid,
    t1iplistid,
    t2iplistid,
    t1maclistid,
    t2maclistid,
    t1portlistid,
    t2portlistid,
    t1scheduleid,
    t2scheduleid,
    t1contextid,
    t2contextid,
    url_link_final,
    tenant1key,
    url_link_final_2,
    tenant2key,
):
    allfirewallrule = []
    allfirewallrulename = []
    allfirewallruleidnew1 = []
    allfirewallruleidold = []
    allfirewallcustomrule = []
    # describe Firewall rules
    log.info("Searching and Modifying Firewall rules in Tenant 1...")
    if firewallruleid:
        for count, dirlist in enumerate(firewallruleid):
            payload = {}
            url = url_link_final + "api/firewallrules/" + str(dirlist)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            allfirewallrule.append(describe)
            firewalljson = json.loads(describe)
            allfirewallrulename.append(str(firewalljson["name"]))
            log.info(
                "#" + str(count) + " Firewall rule name: " + str(firewalljson["name"]),
            )
            log.info("#" + str(count) + " Firewall rule ID: " + dirlist)
    log.info("Done!")
    log.info("Replacing firewall rule IDs configuration in tenant 2...")
    for count, describe in enumerate(allfirewallrule):
        firewalljson = json.loads(describe)
        if "scheduleID" in firewalljson:
            indexnum = t1scheduleid.index(str(firewalljson["scheduleID"]))
            firewalljson["scheduleID"] = t2scheduleid[indexnum]
        if "contextID" in firewalljson:
            indexnum = t1contextid.index(str(firewalljson["contextID"]))
            firewalljson["contextID"] = t2contextid[indexnum]
        if "sourceIPListID" in firewalljson:
            indexnum = t1iplistid.index(str(firewalljson["sourceIPListID"]))
            firewalljson["sourceIPListID"] = t2iplistid[indexnum]
        if "sourceMACListID" in firewalljson:
            indexnum = t1maclistid.index(str(firewalljson["sourceMACListID"]))
            firewalljson["sourceMACListID"] = t2maclistid[indexnum]
        if "sourcePortListID" in firewalljson:
            indexnum = t1portlistid.index(str(firewalljson["sourcePortListID"]))
            firewalljson["sourcePortListID"] = t2portlistid[indexnum]
        if "destinationIPListID" in firewalljson:
            indexnum = t1iplistid.index(str(firewalljson["destinationIPListID"]))
            firewalljson["destinationIPListID"] = t2iplistid[indexnum]
        if "destinationMACListID" in firewalljson:
            indexnum = t1maclistid.index(str(firewalljson["destinationMACListID"]))
            firewalljson["destinationMACListID"] = t2maclistid[indexnum]
        if "destinationPortListID" in firewalljson:
            indexnum = t1portlistid.index(str(firewalljson["destinationPortListID"]))
            firewalljson["destinationPortListID"] = t2portlistid[indexnum]
        describe = json.dumps(firewalljson)
        allfirewallrule[count] = describe
    for count, dirlist in enumerate(allfirewallrulename):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/firewallrules/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST",
            url,
            headers=headers,
            data=payload,
            verify=cert,
        )
        describe = str(response.text)
        taskjson = json.loads(describe)
        if not "message" in taskjson:
            index = describe.find(dirlist)
            if index != -1:
                index = describe.find('"ID"')
                if index != -1:
                    indexpart = describe[index + 4 :]
                    startIndex = indexpart.find(":")
                    if startIndex != -1:  # i.e. if the first quote was found
                        endIndex = indexpart.find(",", startIndex + 1)
                        if (
                            startIndex != -1 and endIndex != -1
                        ):  # i.e. both quotes were found
                            indexid = indexpart[startIndex + 1 : endIndex]
                            allfirewallruleidnew1.append(str(indexid))
                            allfirewallruleidold.append(count)

                            payload = allfirewallrule[count]
                            url = url_link_final_2 + "api/firewallrules/" + str(indexid)
                            headers = {
                                "api-secret-key": tenant2key,
                                "api-version": "v1",
                                "Content-Type": "application/json",
                            }
                            response = requests.request(
                                "POST",
                                url,
                                headers=headers,
                                data=payload,
                                verify=cert,
                            )
                            log.info(
                                "#" + str(count) + " Firewall rule ID: " + indexid,
                            )
                        else:
                            endIndex = indexpart.find("}", startIndex + 1)
                            if (
                                startIndex != -1 and endIndex != -1
                            ):  # i.e. both quotes were found
                                indexid = indexpart[startIndex + 1 : endIndex]
                                allfirewallruleidnew1.append(str(indexid))
                                allfirewallruleidold.append(count)

                                payload = allfirewallrule[count]
                                url = (
                                    url_link_final_2
                                    + "api/firewallrules/"
                                    + str(indexid)
                                )
                                headers = {
                                    "api-secret-key": tenant2key,
                                    "api-version": "v1",
                                    "Content-Type": "application/json",
                                }
                                response = requests.request(
                                    "POST",
                                    url,
                                    headers=headers,
                                    data=payload,
                                    verify=cert,
                                )
                                log.info(
                                    "#" + str(count) + " Firewall rule ID: " + indexid,
                                )
                else:
                    log.info(describe)
                    log.info(payload)
            else:
                allfirewallcustomrule.append(count)
        else:
            log.info(describe)
            log.info(payload)
    log.info("Done!")
    return (
        allfirewallrule,
        allfirewallruleidnew1,
        allfirewallruleidold,
        allfirewallcustomrule,
    )


def FirewallCustom(
    allfirewallrule, allfirewallcustomrule, url_link_final_2, tenant2key
):
    allfirewallruleidnew2 = []
    if allfirewallcustomrule:
        log.info("Creating Firewall Custom Rule...")
        for count, indexnum in enumerate(allfirewallcustomrule):
            payload = allfirewallrule[indexnum]
            url = url_link_final_2 + "api/firewallrules"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find(",", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        allfirewallruleidnew2.append(str(indexid))
                        log.info(
                            "#" + str(count) + " Firewall rule ID: " + indexid,
                        )
                    else:
                        endIndex = indexpart.find("}", startIndex + 1)
                        if (
                            startIndex != -1 and endIndex != -1
                        ):  # i.e. both quotes were found
                            indexid = indexpart[startIndex + 1 : endIndex]
                            allfirewallruleidnew2.append(str(indexid))
                            log.info(
                                "#" + str(count) + " Firewall rule ID: " + indexid,
                            )
            else:
                log.info(describe)
                log.info(payload)
        log.info("Done!")
    return allfirewallruleidnew2


def FirewallReplace(
    allofpolicy,
    allfirewallruleidnew1,
    allfirewallruleidnew2,
    firewallruleid,
    allfirewallruleidold,
    allfirewallcustomrule,
    t1statefulid,
    t2statefulid,
):
    if allofpolicy:
        for count, describe in enumerate(allofpolicy):
            taskjson = json.loads(describe)
            indexid = taskjson["firewall"]
            if "globalStatefulConfigurationID" in taskjson["firewall"]:
                indexnum = t1statefulid.index(
                    str(taskjson["firewall"]["globalStatefulConfigurationID"])
                )
                taskjson["firewall"]["globalStatefulConfigurationID"] = t2statefulid[
                    indexnum
                ]
            if "ruleIDs" in taskjson["firewall"]:
                if allfirewallruleidnew1 or allfirewallruleidnew2:
                    for count1, this in enumerate(taskjson["firewall"]["ruleIDs"]):
                        checkindex = firewallruleid.index(str(this))
                        if checkindex in allfirewallruleidold:
                            checkindex1 = allfirewallruleidold.index(checkindex)
                            taskjson["firewall"]["ruleIDs"][
                                count1
                            ] = allfirewallruleidnew1[checkindex1]
                        elif checkindex in allfirewallcustomrule:
                            checkindex1 = allfirewallcustomrule.index(checkindex)
                            taskjson["firewall"]["ruleIDs"][
                                count1
                            ] = allfirewallruleidnew2[checkindex1]
            allofpolicy[count] = json.dumps(taskjson)
    return allofpolicy



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dsmigrator']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'certifi>=2020.12.5,<2021.0.0',
 'click>=7.1.2,<8.0.0',
 'deep-security-api>=20.0.60,<21.0.0',
 'nested-lookup>=0.2.22,<0.3.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'requests>=2.25.1,<3.0.0',
 'rich>=10.0.0,<11.0.0',
 'six>=1.15.0,<2.0.0',
 'urllib3>=1.26.4,<2.0.0',
 'zeep>=4.0.0,<5.0.0']

entry_points = \
{'console_scripts': ['dsmg = dsmigrator.__main__:main',
                     'dsmg-rules = dsmigrator.rules_migrator:main']}

setup_kwargs = {
    'name': 'dsmigrator',
    'version': '0.5.0',
    'description': 'A cli tool to migrate Trend Micro Deep Security to the cloud.',
    'long_description': '<p align="center">\n  <img src="./logo.png" />\n</p>\n\n<h1 align="center" style="border-bottom: none">Trend Micro Policy Migrator</h1>\n<p align="center">\n  Moves your existing on-prem Deep Security deployment to CloudOne Workload Security.\n</p>\n<p align="center">Automatically.</p>\n\n<details>\n  <summary>TABLE OF CONTENTS</summary>\n\n- [Quickstart](#quickstart)\n- [Capabilities](#capabilities)\n  - [Known limitations](#known-limitations)\n- [Usage](#usage)\n  - [Command Reference](#command-reference)\n  - [Use Environment Variables](#use-environment-variables)\n- [Requirements](#requirements)\n- [Contributing](#contributing)\n- [Support](#support)\n- [License](#license)\n</details>\n\n## Quickstart\n\nUse the package manager [pip](https://pip.pypa.io/en/stable/) to install dsmigrator.\n\n1. Run ```pip install dsmigrator``` on a machine with access to your DSM.\n\n2. Run ```dsmg -k``` and fill out the credential prompts.\n\n## Capabilities\n\nHere\'s the current feature map of what the tool can migrate:\n\n- [x] Policies\n- [x] Policy settings\n- [x] Anti-Malware Scan Configurations\n- [x] IPS, LI, and IM custom rules\n- [x] Firewall rules\n- [x] Schedules\n- [x] Contexts\n- [x] IP lists\n- [x] MAC lists\n- [x] Port lists\n- [x] [BETA] Tasks (still quite buggy)\n- [x] [BETA] Computer Groups\n- [ ] Application Control (everything)\n- [ ] Self-signed certificate support for authenticated requests\n\n### Known limitations\n\n- Cannot migrate customized IM/LI/IP rules. Another tool will be incoming to help aid a manual process in identifying each rule that has been customized, but they will never migrate automatically due to an API limitation\n- Won\'t migrate cloud accounts. Must be reconfigured/reauthenticated in Cloud One\n- Doesn\'t migrate DSM settings, make sure to check these manually.\n- Application Control support is not on the roadmap currently. Please open an issue if this is \n\n## Usage\n\n### Command Reference\n\n```text\nUsage: dsmg [OPTIONS]\n\n  Moves your on-prem DS deployment to the cloud!\n\nOptions:\n  -ou, --original-url TEXT        A resolvable FQDN for the old DSM, with port\n                                  number (e.g. https://192.168.1.1:4119/)\n\n  -oa, --original-api-key TEXT    API key for the old DSM with Full Access\n                                  permissions\n\n  -nu, --new-url TEXT             Destination url  [default:\n                                  https://cloudone.trendmicro.com/]\n\n  -coa, --cloud-one-api-key TEXT  API key for Cloud One Workload Security with\n                                  Full Access permissions\n\n  -d, --delete-policies / --keep-policies\n                                  Wipes existing policies in Cloud One (not\n                                  required, but will give best results)\n\n  -t, --tasks                     (BETA) Enable the task migrator (may be\n                                  buggy)\n\n  -k, --insecure                  Suppress the InsecureRequestWarning for\n                                  self-signed certificates\n\n  -f, --filter TEXT               A list of policy names in form \'[name, name,\n                                  ...]\' which are the only ones which will be\n                                  transferred.\n\n  --help                          Show this message and exit.\n```\n### Use Environment Variables\n\nYou can optionally use the following environment variables to pass in your credentials:\n\n- `ORIGINAL_API_KEY`\n- `ORIGINAL_URL`\n- `CLOUD_ONE_API_KEY`\n\n## Requirements\n\n- Python3 (only tested on Python 3.7 or greater so far, so your mileage may vary)\n- One api key for your old Deep Security Manager with "Full Access" permissions\n- One api key for your Cloud One account with "Full Access" permissions\n- A resolvable FQDN to your old Deep Security Manager\n\n**NOTE:** DS Migrator currently only supports migrations from Deep Security 20 and 12.\n\n## Contributing\n\n1. Run ./dev-setup.sh, which will download nix and nix flakes.\n2. Run `nix develop` which will download and build dependencies, and drop you in a shell.\n\n## Support\n\nFor support, please open an issue on Github.\n\n## License\n\nGNU General Public License\n',
    'author': 'Alex Jackson',
    'author_email': 'alex_jackson@trendmicro.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ajaxbits/ds-migrator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import sys
import os
import requests
import urllib3
import urllib3
import json
from dsmigrator.logging import log
from dsmigrator.api_config import ComputerGroupsApiInstance
from dsmigrator.migrator_utils import validate_create_dict, rename_json
from deepsecurity.rest import ApiException

cert = False


def computer_group_listmaker(OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    computer_group_dict = {}
    enum_oldetname, groupIDs = ListGroup(OLD_HOST, OLD_API_KEY)
    if groupIDs:
        allgroup, namegroup = GetGroup(groupIDs, OLD_HOST, OLD_API_KEY)
        computer_group_dict = CreateGroup(allgroup, namegroup, NEW_HOST, NEW_API_KEY)
    computer_group_dict[0] = 0
    return computer_group_dict


def ListGroup(url_link_final, tenant1key):
    payload = {}
    url = url_link_final + "api/computergroups"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request(
        "GET",
        url,
        headers=headers,
        data=payload,
        verify=cert,
    )
    describe = str(response.text)
    oldetname = []
    oldetid = []
    ebt_json = json.loads(describe).get("computerGroups")
    if ebt_json is not None:
        for here in ebt_json:
            oldetname.append(str(here["name"]))
            oldetid.append(str(here["ID"]))
    return enumerate(oldetname), oldetid


def GetGroup(etIDs, url_link_final, tenant1key):
    allet = []
    nameet = []
    log.info("Getting Target Task...")
    if etIDs:
        for part in etIDs:
            payload = {}
            url = url_link_final + "api/computergroups/" + str(part)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                data=payload,
                verify=cert,
            )
            describe = str(response.text)
            allet.append(describe)
            namejson = json.loads(describe)
            nameet.append(str(namejson["name"]))
        return allet, nameet


def CreateGroup(allet, nameet, url_link_final_2, tenant2key):
    log.info("Creating group to target Account...")
    if nameet:
        modet = []
        for task in allet:
            task_json = rename_json(json.loads(task))
            modet.append(json.dumps(task_json))
        return validate_create_dict(
            modet, ComputerGroupsApiInstance(tenant2key), "Computer Group"
        )
